clear variables; clc;
x0 = [0.1 50 0 0 0 0 0 0.1 50 0 0 0]; %initial values
% Model for BuB is in mol/L. Need toconvert to g/L
MW.buoh = 74.122; % [g/mol]
MW.hbu = 88.1; % [g/mol]
MW.bub = 144.21; % [g/mol]
%Coefficient matrices for the models
B = [0.678 30 0.06 0.75 3.53 0.272 0.3395 2.26 0.793 0.01 0.951 0.0861 0.1 0.01372]; %butanol fermenter coefficient matrix 
C = [0.48 1.62 0.807 0.978 0.999 2.92 0.057 0.72 0]; %butyric acid fermenter and butyl butyrate esterification coefficient matrix 
D = [0.02, 0.051, 0.228, 0.033];%*1e-3;% Butyl butyrate coefficient matrix [mol/(L min g_enzyme), mol/L, mol/L, mol/L] [rmax, Kma, Kmb, Kib]. 1e-3 converts values from mmol to mol

tspan.hbu = [0 40];
%tspan.buoh = tspan.hbu;
tspan.buoh = [0, 20];
%tspan.bub = [tspan.hbu(end), tspan.hbu(end) + 10];
tspan.bub = [0, 20];

%solve buoh system
[t.buoh,x.buoh] = ode45(@(t,x) kineticSystemBuOH(t,x,B), tspan.buoh, x0);
%solve hbu system
[t.hbu,x.hbu] = ode45(@(t,x) kineticSystemHBu(t,x,C), tspan.hbu, x0);

%update input to bub-block
x0([5, 10]) = [x.buoh(end, 5), x.hbu(end, 10)];
%update r_max in D-block to include the grams of enzyme required
D(1) = D(1)*60*160e-3*(x0(5)/(.01*MW.buoh/15e-3)); % [mol/(L min g_enzyme)] -> [mol/(L h)]. They used 160mg of enzyme for 0.01 mol in 15ml solution (xf(5)/((.01/15e-3)*MW.buoh))*160e-3

%solve bub system
[t.bub,x.bub] = ode45(@(t,x) kineticSystemBuB(t,x,D, MW), tspan.bub, x0);


% % to find optimal substrate ratio %I didn't get it working yet
% % s1 + s2 = s --> ratio = s1/s
% ratio=(0.1:0.1:1);
% n=length(ratio);
% x12=cell(1,n); %since ode45 is a variable step solver, you don't know how many t and x values you'll get each time, so the simplest solution would be to save them all in cell arrays 
% allt=cell(1,n);
% BuB=cell(1,n);
% 
% for i=1:n
% [t,x] = ode45(@(t,x) Scenario1(t,x,B,C,D), tspan, x0); % one cell array for each ratio value aka ratio(i)
% allt{i}=t;
% x12{i}=x(:,12); %BuB
% BuB{i}=x12{i}(end); %final BuB conc. (max)
% end
% 
% figure
% plot(ratio,cell2mat(BuB));
% xlabel('ratio')
% ylabel('Maximum BuB concentration')

% figure
% plot(t,x(:,1)); %biomass 1
% grid on
% ylabel('Concentration (g/l)')
% xlabel('Time (h)')
% title('Biomass1')
% 
% figure 
% plot(t,x(:,2)); %glucose 1
% grid on
% ylabel('Concentration (g/l)')
% xlabel('Time (h)')
% title('Glucose1')
% % 
% figure
% plot(t,x(:,8)); %biomass 2
% grid on
% ylabel('Concentration (g/l)')
% xlabel('Time (h)')
% title('Biomass2')
% 
% figure
% plot(t,x(:,9)); %glucose 2
% grid on
% ylabel('Concentration (g/l)')
% xlabel('Time (h)')
% title('Glucose2')


figure
subplot(3,1,1)
plot(t.buoh,x.buoh(:,5)); %butanol
grid on
ylabel('Concentration (g/l)')
xlabel('Time (h)')
title('Butanol block')

subplot(3,1,2)
plot(t.hbu,x.hbu(:,10), 'r'); %butyric acid
hold on
plot(t.hbu,x.hbu(:,11), 'g'); %Acetic acid
grid on
ylabel('Concentration (g/l)')
xlabel('Time  (h)')
legend('HBu', 'HAc')
title('Butyric acid block')

subplot(3,1,3)
ax = gca;
plot(t.bub, x.bub(:, 5)) %butanol
hold on
plot(t.bub, x.bub(:, 10)); %butyric acid
plot(t.bub,x.bub(:,12)); %butyl butyrate

grid on
ylabel('Concentration (g/l)')
xlabel('Time (h)')
legend([{'Butanol'}, {'Butyric acid'}, {'Butyl butyrate'}], 'box', 'on')
title('BuB block')
xl = xlim;
xlim([0, xl(end)])

ha = axes('Position',[0 0 1 1],'Xlim',[0 1],'Ylim',[0 ... 
1],'Box','off','Visible','off','Units','normalized', 'clipping' , 'off');

text(0.5, 1,'\bf Model in separate blocks','HorizontalAlignment' ...
,'center','VerticalAlignment', 'top')
% figure 
% plot(t,x(:,12)); %butyl butyrate
% grid on
% ylabel('Concentration (g/l)')
% xlabel('Time (h)')
% legend('Butyl butyrate')
