import numpy as np 
import matplotlib as mpl  
mpl.use('Qt5Agg') #works as an interactive backend. There is currently a bug with the backend macosx
# which makes it not possible to pan 3D plots
import matplotlib.pyplot as plt
# import matplotlib.cm #color map
# import matplotlib.patches
# import matplotlib.lines
from mpl_toolkits.mplot3d import Axes3D
import os
import numpy.matlib
import sys
# import os.path
homeDir = os.path.expanduser('~')
sys.path.insert(0, homeDir + '/Dropbox/Masteroppgave/PythonFiles')
import artools
from kineticsBub import rxRateFun, solveMoleBalance#, rxRateAllNumpy


# #Make/check folders for storage of the dictionaries which will be computed
# dirmain = os.getcwd()
# dirCSV = dirmain + '/computedCSV'
# if not os.path.exists(dirCSV):
# 	os.makedirs(dirCSV)
# dirPlots = dirmain + '/plots'
# if not os.path.exists(dirPlots):
# 	os.makedirs(dirPlots)


def getARbub(feedToBub, numPointsPFR = 300):
	#Declare initial data

	#indices in arrays
	ibuoh = 0
	ihbu = 1
	ibub = 2
	isub = 3
	#ihac
	#ietoh
	itau = 4
	#feed concentration, according to indices declared above
	cfAll = feedToBub #np.array([49.4147, 58.7333, 0, 10, 0])#g/L. Feed concentrations (last is tau [h])
	cf = np.array([cfAll[ibub], cfAll[itau]])# [g/l, h]. State space vector
	# print(cfAll)
	# print(cf)
	MW = np.array([74.122, 88.1, 144.21, 180.1559]) #[g/mol] [buoh, hbu, bub, s]
	rc = {'rmax': 0.02*60*160e-3, #mol/(l min g_enzyme) * min/h * g_enz (used 160g enzyeme fr 0,01mol buoh/hbu in 15ml sol)
		'Kma': 0.051, #mol/L
		'Kmb': .228, #mol/L
		'Kia': .033, #mol/L
		'cf': cf, 'cfAll': cfAll, # g/L
		'MW': MW, # g/mol
		'isub': isub, 'ihbu': ihbu,'ibuoh': ibuoh, 'ibub': ibub, 'itau': itau} #indices
	tEnd = {'PFR': 20, 'CSTR': 20}

	#Make dicts for saving the data from AR calculations
	cPFR = {}
	tPFR = {}
	cCSTR = {}
	tCSTR = {}
	cPFRall = {} #include for all components
	cCSTRall = {}

	#Calculated the PFR trajectories and CSTR locus
	rxRate = lambda conc: rxRateFun(conc, rc)
	cPFR['f'], tPFR['f'] = artools.PFRtrajectory(rxRate, cf, tEnd['CSTR'], numPoints = numPointsPFR) 
	cPFRall['f'] = solveMoleBalance(cPFR['f'], rc)

	cCSTR['f'], tCSTR['f'] = artools.CSTRlocusFast(rxRate, cf, tEnd['CSTR'], numPoints = numPointsPFR)#Be sure to include eq (tend=1e4 is ok
	cCSTRall['f'] = solveMoleBalance(cCSTR['f'], rc)


	"""
	Make and draw the convex hull. Make sure that the hull is containing a vertical line 
	from (max(X),y_data)-(max(X),max(ylim)) and from (0,0)-(0, max(ylim))
	Steps: 
		1) Gather all data
		2) Compute the convex hull
		3) Plot the convex hull
		
	"""
	#data gathering
	cDataAll = cf
	for key in cPFR.keys():
		cDataAll = np.vstack((cDataAll, cPFR[key]))
	for key in cCSTR.keys():
		cDataAll = np.vstack((cDataAll, cCSTR[key]))

	maxXdata = np.amax(cDataAll[:, 0])
	maxYlim = tEnd[max(tEnd, key=tEnd.get)]*1.1 #increase the time with extra 10% #ax['AR'].get_ylim()[1]

	maximumPoints = np.array([[maxXdata, maxYlim],
		[cf[0], maxYlim]])
	cDataAll = np.vstack((cDataAll, maximumPoints))#add the new points

	#computation
	hull = artools.convhull(cDataAll, incrementalBool = True)
	makeOptimCSTRindexPlot = False
	if makeOptimCSTRindexPlot:
		optimCSTRIndex, fig['optimCSTRIndex'], ax['optimCSTRIndex'] = artools.optimCSTRforTau2D(hull, 
			cCSTR['f'], rxRate, tEnd['PFR'], makePlot = makeOptimCSTRindexPlot)
	else:
		optimCSTRIndex = artools.optimCSTRforTau2D(hull, cCSTR['f'], rxRate, tEnd['PFR'], 
			makePlot = makeOptimCSTRindexPlot)

	#Return the variables
	if optimCSTRIndex == len(cCSTR['f'][:, 0]) - 1: #We're at equilibrium, this is a trivial solution
		optimCSTRIndex = []
		cPFRall['2'] = []
		return cPFRall['f'], cCSTRall['f'], optimCSTRIndex, cPFRall['2']
	else:
		# Make the new PFR trajectory from the optimal CSTR outlet concentration. Also expand the plot
		cPFR['2'], tPFR['2'] = artools.PFRtrajectory(rxRate, cCSTR['f'][optimCSTRIndex, :], 
			tEnd['PFR'] - cCSTR['f'][optimCSTRIndex, 1])
		cPFRall['2'] = solveMoleBalance(cPFR['2'], rc)
		return cPFRall['f'], cCSTRall['f'], optimCSTRIndex, cPFRall['2']


