import numpy as np 
import matplotlib as mpl  
mpl.use('Qt5Agg') #works as an interactive backend. There is currently a bug with the backend macosx
# which makes it not possible to pan 3D plots
import matplotlib.pyplot as plt
# import matplotlib.cm #color map
# import matplotlib.patches
# import matplotlib.lines
from mpl_toolkits.mplot3d import Axes3D
import os, shutil
import numpy.matlib
import sys
homeDir = os.path.expanduser('~')
sys.path.insert(0, homeDir + '/Dropbox/Masteroppgave/PythonFiles')
import artools
import getARbub
import kineticsBub
dirMain = os.getcwd()

font = {'size'   : 12}
mpl.rc('font', **font)

#Make plot folder
dirPlots = dirMain + '/plots'
if not os.path.exists(dirPlots):
	os.makedirs(dirPlots)


#define dicts where to store info
buoh = {}
hbu = {}


#direction to csv folders
dirCSV = {}
dirCSV['buoh'] = homeDir + '/Dropbox/Masteroppgave/PythonFiles/bioProcessButylButyrate/scenario2-sep blocks/butanol/computedCSV'
dirCSV['hbu'] = homeDir + '/Dropbox/Masteroppgave/PythonFiles/bioProcessButylButyrate/scenario2-sep blocks/hbuBlock/computedCSV'
dirCSV['resBub'] = os.getcwd() + '/resBub'
if not os.path.exists(dirCSV['resBub']):
	os.makedirs(dirCSV['resBub'])

dirCSV['bub'] = dirMain + '/cBubForMainARfile'
if not os.path.exists(dirCSV['bub']): # directory for saving the computed bub-concentrations (initiated from the grid points)
	os.makedirs(dirCSV['bub'])

## Import the data. Only use states which are on the boundary of the AR, that is the CSTR + 2nd PFR
buoh['PFR']= np.genfromtxt(dirCSV['buoh'] + '/dataPFRallf.csv', delimiter = ',') 
buoh['CSTR'] = np.genfromtxt(dirCSV['buoh'] + '/dataCSTRall.csv', delimiter = ',')
print(buoh['PFR'][-1,:])
print(buoh['CSTR'][-1,:])

hbu['PFR']= np.genfromtxt(dirCSV['hbu'] + '/dataPFRall2.csv', delimiter = ',') 
hbu['CSTR'] = np.genfromtxt(dirCSV['hbu'] + '/dataCSTRall.csv', delimiter = ',')
hbu['optimIndex'] = int(np.genfromtxt(dirCSV['hbu'] + '/optimCSTRindex.txt'))

# Make data points along the mixing lines
buoh['mix'] = np.array([(1-i)*buoh['PFR'][0, :] + i*buoh['PFR'][-1, :] 
	for i in np.linspace(0,1)]) #CSTR locus is the optimal here, this is a special case
hbu['mix'] = np.array([(1-i)*hbu['CSTR'][0, :] + i*hbu['CSTR'][hbu['optimIndex'], :] 
	for i in np.linspace(0,1)])

## Obtain data for the boundaries of the feed region to bub block
buoh['boundaryAll'] = np.vstack((buoh['mix'], 
	buoh['PFR']))

hbu['boundaryAll'] = np.vstack((hbu['mix'], 
	hbu['PFR']))


#select the data in the boundaries which are of interest
buoh['boundary'] = np.zeros((buoh['boundaryAll'].shape[0], 3)) #[x,s,hbu,buoh,etoh] indices in buondarAll
buoh['boundary'][:, 0] = buoh['boundaryAll'][:, 3] # cBuOH [g/L]
# buoh['boundary'][:, 1] = buoh['boundaryAll'][:, 2] #cHBu [g/L]. Approximate to 0?
buoh['boundary'][:, 2] = buoh['boundaryAll'][:, 1] # cs [g/L]

hbu['boundary'] = np.zeros((hbu['boundaryAll'].shape[0], 3)) #[s,x,hac,hbu,tau]
# hbu['boundary'][:, 0] equals concentration of BuOH, which is already set to 0
hbu['boundary'][:, 1] = hbu['boundaryAll'][:, 3] # cHBu [g/L]
hbu['boundary'][:, 2] = hbu['boundaryAll'][:, 0] #cS [g/L]


#obtain the convex hull for the region
cDataAllInt = np.vstack((buoh['boundary'], hbu['boundary'])) # store all relevant data in same array
hull = {} # dict if several hull are to be computed
hull['s'] = artools.convhull(cDataAllInt) # key 's' refers to cS on z-axis in plot


## make the 2d projection in (c_buoh - c_hbu)-space
projection = {}
projection['s'] = np.array([buoh['boundary'][0, :], #"feed point" for both buoh and hbu
	buoh['boundary'][-1, :], # end point of buoh
	hbu['boundary'][-1, :]]) # end point for hbu
projection['s'][:, -1] = 0 #set cS = 0 in the projection

#Export the projection in a .csv-file. This file will be read by MATLAB, a grid which is
# inside the projection (by using the MATLAB function inpolygon) will be made and the be exported
# to this file again

numPointsGridTot = 40 #Number of points in each direction in the grid in the projection (15x15 points)
np.savetxt(dirCSV['resBub'] + '/projectionBuohHBu.csv', projection['s'], delimiter = ',')
with open(dirCSV['resBub'] + '/numPointsGrid.txt', 'w') as f:
	f.write(str(numPointsGridTot))
f.close()

#Run MATLAB script now (makeGridInProjection.m, name of the file atm)

#Import the grid (obtained from matlab)
Xm = np.genfromtxt(dirCSV['resBub'] + '/Xm.csv', delimiter = ',')
Ym = np.genfromtxt(dirCSV['resBub'] + '/Ym.csv', delimiter = ',')
Zm = np.genfromtxt(dirCSV['resBub'] + '/Zm.csv', delimiter = ',')
XmSel = Xm[1:, 1:] # Select the part of the mesh which is interesting (i.e not along x and y-axis)
YmSel = Ym[1:, 1:]
numPointsGrid = XmSel.shape[0] #know Xm,Ym,Zm are quadratic and of same shape


"""
In order to relate the grid to the other plots, it is required that the mixing ratio, lambda or 
lambdaPure is obtained. Strategy:
	1) Use the equation for the plane/triangle to obtain cS for every point in the grid
	2) Explot the linear relationship between cs and cbuoh/chbu from the outlet of the previous
		block (can show that R^2 = 1). Make a linear regression such that for every cS, it is 
		possible to obtain cbuoh or chbu from the outlet of the previous block (these points have
		no (direct) correlation to the points in the grid)
	3) Calculate the length from the grid point to each of the "pure" components/outlet of previous 
		block. Pythagoras was used for this (cS for each point is constant and known, operates in 2D)
	4) Calculate mixing ratio from the usual equaiton: lambda = dist1/(dist1 + dist2)

	FLAW: The mixing points do not need to reside in the same 2D plane (don't need to have same cS).
		So this is not exactly true, but the easiest way to map the mesh from one plot to another?


"""
# Relate the grid to the roof of the boundary

#data points for the roof (a triangle, requires only 3 data points)
roofS = np.array([buoh['boundary'][0, :],#the same as hbu['boundary'][0,:], namely the feed point
	buoh['boundary'][-1, :],
	hbu['boundary'][-1, :]])

# Corresponding equation for the surface/triangle
eq = {} 
eq['roofS'] = lambda cbuoh, chbu: buoh['boundary'][0, 2]*(1 - cbuoh/buoh['boundary'][-1, 0] - 
	chbu/hbu['boundary'][-1, 1]) #= cSmax*(1- cbuoh/cbuohMax - chbu/chbuMax)

# #Write equations for cBuoh = a1 + b1*cS and cHBu = a2 + b2*cS (they are linear)

# #obtain the linear equations. First, cBuoh = a + b cS
AbuohEq = np.vstack([buoh['boundary'][:, 2], np.ones(len(buoh['boundary'][:, 2]))]).T#follow the example on doc for np.linalg.lstsq
bBuOH, aBuOH = np.linalg.lstsq(AbuohEq, buoh['boundary'][:, 0])[0]
eq['buohFromS'] = lambda concS: aBuOH + bBuOH*concS
#Then, cHBu = a + b*cS
AhbuEq = np.vstack([hbu['boundary'][:, 2], np.ones(len(hbu['boundary'][:, 2]))]).T#follow the example on doc for np.linalg.lstsq
bHBu, aHBu = np.linalg.lstsq(AhbuEq, hbu['boundary'][:, 1])[0]
eq['hbuFromS'] = lambda concS: aHBu + bHBu * concS



#plot the region
fig = {}
ax = {}

fig['boundary'] = plt.figure('boundary')
ax['boundary'] = fig['boundary'].add_subplot(111, projection = '3d')
ax['boundary'].set_xlabel(r'$C_{BuOH}$ [g/L')
ax['boundary'].set_ylabel(r'$C_{HBu}$ [g/L]')
ax['boundary'].set_zlabel(r'$C_S$ [g/L]')

#plot the oundaries
ax['boundary'].plot(buoh['boundary'][:, 0], buoh['boundary'][:, 1], buoh['boundary'][:, 2], 
	c = 'b', label = 'BuOH') # AR for buoh process
ax['boundary'].plot(hbu['boundary'][:, 0], hbu['boundary'][:, 1], hbu['boundary'][:, 2], 
	c = 'g', label = 'HBu') # AR for hbu process

#Plot the convex hull
ax['boundary'].plot_trisurf(cDataAllInt[:, 0], cDataAllInt[:, 1], cDataAllInt[:, 2], color = 'r', 
	edgecolor = None, alpha = .2, triangles = hull['s'].simplices) #plot the convex hull
# Projection of the AR
ax['boundary'].plot_trisurf(projection['s'][:, 0], projection['s'][:, 1], projection['s'][:, 2], 
	color = 'k', alpha = .2)
# Mesh in the projection
ax['boundary'].scatter(XmSel, YmSel, color = 'k', s = 3, marker = 'x')
# Insert the three points defining the triangle/roofS
ax['boundary'].scatter(roofS[:, 0], roofS[:, 1], roofS[:, 2], c = 'k', marker = '^')


#Make the AR plot. Make it in BuOH-S-BuB space
fig['AR'] = plt.figure('AR')
ax['AR'] = fig['AR'].add_subplot(111, projection = '3d')
ax['AR'].set_xlabel(r'$C_{BuOH}$ [g/L')
ax['AR'].set_ylabel(r'$C_S$ [g/L]')
ax['AR'].set_zlabel(r'$C_{BuB}$ [g/L]')

#the feed region
ax['AR'].plot(buoh['boundary'][:, 0], #cBuOH
	buoh['boundary'][:, 2], #cS
	np.zeros(buoh['boundary'][:,0].shape), #cBuB
	color = 'b', label = 'BuOH')

ax['AR'].plot(hbu['boundary'][:, 0], #cBuOH
	hbu['boundary'][:, 2], #cS
	np.zeros(hbu['boundary'][:,0].shape), #cBuB
	color = 'm', label = 'HBu')


cs = np.zeros(XmSel.shape) #store corresponding cs to each feed point to bub
cs.fill(np.nan) #initialize array to be filled with nan
lambdaPure = np.zeros(XmSel.shape) #same here; store lambda parameter for every point in the mesh
lambdaPure.fill(np.nan)
buohBlockPureMesh = {} # store a np.array([cbuohOut(cs), 0, cs]), where cs is the sugar 
# concentration in the grid and cbuohOut(cs) is the corresponding buoh 
# concentration (concentrations resulting from the outlet of the buoh block)
hbuBlockPureMesh = {} #same here; corresponding concentration from the outlet of the hbu-block
cf2bub = {} #save all the feed points to the bub block
cBubFinalAll = np.zeros((numPointsGrid, numPointsGrid)) #store final (end) concentrations for all cf2bub
cBubFinalAll.fill(np.nan)
cBubBlock = {}
calcBubConc = False #redo calculations for bub-calculations
cPFRall = {}
cCSTRall = {}
feedRegion2bub = {}
feedRegion2bub['buoh'] = np.column_stack((buoh['boundary'][:, 0], buoh['boundary'][:, 1], 
	np.zeros(buoh['boundary'][:, 0].shape), buoh['boundary'][:, 2], 
	np.zeros(buoh['boundary'][:, 0].shape))) #last is tau, don't have data for this?
feedRegion2bub['hbu'] = np.column_stack((hbu['boundary'][:, 0], hbu['boundary'][:, 1],
	np.zeros(hbu['boundary'][:, 0].shape), hbu['boundary'][:, 2], 
	np.zeros(hbu['boundary'][:, 0].shape)))
cDataAllBub = np.vstack((feedRegion2bub['buoh'], feedRegion2bub['hbu']))

if calcBubConc:
	optimCSTRindex = []
	#delete all the current files in the folder. This is necessary if a different grid size is used
	for the_file in os.listdir(dirCSV['bub']):
		file_path = os.path.join(dirCSV['bub'], the_file)
		try:
			if os.path.isfile(file_path):
				os.unlink(file_path)
			#elif os.path.isdir(file_path): shutil.rmtree(file_path)
		except Exception as e:
			print(e)
else:
	optimCSTRindex = np.genfromtxt(dirCSV['bub'] + '/optimCSTRindex.csv', delimiter = ',')

for i in range(numPointsGrid):
	buohBlockPureMesh[i] = {}
	hbuBlockPureMesh[i] = {}
	cf2bub[i] = {}
	cBubBlock[i] = {}
	cPFRall[i] = {}
	for j in range(numPointsGrid):
		feedToBub = np.array([XmSel[i, j], YmSel[i, j]]) #np.array([buoh, hbu])
		if not np.isnan(feedToBub).any(): #check if any of the numbers in the feed is nan. If they 
		# are, the feed point is invalid (do nothing)
			cs[i, j] = eq['roofS'](feedToBub[0], feedToBub[1]) #obtain cS for a grid point.
			if cs[i, j] < 0:
				# print('Negative cs in grid from roofEq: ' + str(cs[i,j]) + '. Value changed to 0')
				cs[i, j] = 0
			#obtain the pure concentrations of BuOH and HBu in order to determine the lambda parameter (
			# the relative distance of the grid point to each of the sides)
			cBuOHpure = eq['buohFromS'](cs[i, j])
			cHBuPure = eq['hbuFromS'](cs[i, j])
			dist2BuOH = np.sqrt((feedToBub[0] - cBuOHpure)**2 + (feedToBub[1] - 0)**2)
			dist2HBu= np.sqrt((feedToBub[0] - 0)**2 + (feedToBub[1] - cHBuPure)**2)
			lambdaPure[i, j] = dist2BuOH / (dist2BuOH + dist2HBu) #mixing parameter, lambda. The
			# lambdaPure = V_HBu/(V_HBu + V_BuOH), (short dist2buoh indicats V_BuOH >> V_HBu, but lambda << .5)
			#Check if the procedure is correct: calculate the mesh based on the lambda parameter
			buohPurePoint = np.array([cBuOHpure, 0, cs[i, j]])  
			hbuPurePoint = np.array([0, cHBuPure, cs[i, j]])
			buohBlockPureMesh[i][j] = np.array([cBuOHpure, 0, cs[i, j]])  
			hbuBlockPureMesh[i][j] = np.array([0, cHBuPure, cs[i, j]])
			cMix = (1 - lambdaPure[i, j])*buohPurePoint + lambdaPure[i, j]*hbuPurePoint
			# cMix = (lambdaPure[i, j])*buohPurePoint + (1-lambdaPure[i, j])*hbuPurePoint
			ax['boundary'].scatter(feedToBub[0], feedToBub[1], cs[i, j], color = 'r', marker = 'o')
			ax['boundary'].scatter(cMix[0], cMix[1], cMix[2], color= 'g', marker = '^', s = 30)
			# plt.draw() #check if the cMix procedure is correct (does red and green markers 
			# # come on top of each other at the same time?)
			# plt.pause(1e-5)

			#Map the grid from the 'boundary'-plot to the 'AR'-plot. First, make
			# the feed to the bub-block
			cf2bub[i][j] = np.array([cMix[0], cMix[1], 0, cMix[2], 0]) #[cBuOH, cHBu, cBuB, cS, tau], 
			# this is the feed 2 bub-block
			if calcBubConc: # recalcualte and save output to .csv-files. Takes some time
				cPFRall[i][j], cCSTRall['f'], optimCSTRindexNew, cPFR2 = getARbub.getARbub(cf2bub[i][j])
				if optimCSTRindexNew == []:
					optimCSTRindexNew = 0
				# print(optimCSTRindexNew)
				optimCSTRindex.append(optimCSTRindexNew)
				# print(optimCSTRindex)
				np.savetxt(dirCSV['bub'] + '/cPFRall-' + str(i) + '-' + str(j) + '.csv', 
					cPFRall[i][j], delimiter = ',')
				np.savetxt(dirCSV['bub'] + '/cCSTRall-' + str(i) + '-' + str(j) + '.csv', 
					cCSTRall['f'], delimiter = ',')

			else: # Get the data from the CSV files
				cPFRall[i][j] = np.genfromtxt(dirCSV['bub'] + '/cPFRall-' + str(i) + 
					'-' + str(j) + '.csv', delimiter = ',')

			#Incorporate the PFR data into the cDataAllBub
			cDataAllBub = np.vstack((cDataAllBub, cPFRall[i][j]))

			# Obtain max concentration of bub
			cBubFinalAll[i, j] = max(cPFRall[i][j][:, 2]) 

			# Plot the bub-data (first, the feed data)
			ax['AR'].scatter(cf2bub[i][j][0], #cfBuOH
				cf2bub[i][j][3], #cfS
				cf2bub[i][j][2], #cfBuB
				color= 'g', marker = '^', s = 30)
			# Plot the bub solution trajectories
			ax['AR'].plot(cPFRall[i][j][:, 0], cPFRall[i][j][:, 3], cPFRall[i][j][:, 2],
				color = 'g', linewidth = .5)
			if calcBubConc: # procedure takes time, nice to visualize how long it is left
				plt.draw()
				plt.pause(1e-8)
if calcBubConc: #need to save the optimCSTRindex for the newly calculated data
	# optimCSTRindex = np.asarray()
	np.savetxt(dirCSV['bub'] + '/optimCSTRindex.csv', optimCSTRindex, delimiter = ',')
		
#Obtain and plot the convex hull
# print(cDataAllBub[:, [0, 3, 2]].shape)
hull['AR'] = artools.convhull(cDataAllBub[:, [0, 3, 2]]) # [0,3,2] correspond to [buoh,s,bub]
# ax['AR'].plot_trisurf(cDataAllBub[:, 0], cDataAllBub[:, 3], cDataAllBub[:, 2], color = 'r',
# 	edgecolor = None, alpha = .2, triangles = hull['AR'].simplices)

#Insert a legend for the Bub-block
ax['AR'].plot([np.nan, np.nan], [np.nan, np.nan], [np.nan, np.nan], color = 'g', label = 'BuB',
	linewidth = .5)



# Obtain the indices of the maximum concentrations
# i,j = np.unravel_index(cBubFinalAll.argmax(), cBubFinalAll.shape)
i,j = np.unravel_index(np.nanargmax(cBubFinalAll), cBubFinalAll.shape)
print('Optimal feed to bub-block:')
print('\t BuOH: %.2f [g/L] <=> %.2f [mmol/L]' % (cf2bub[i][j][0], 1e3*cf2bub[i][j][0]/74.122))
print('\t HBu: %.2f [g/L] <=> %.2f [mmol/L]' % (cf2bub[i][j][1], 1e3*cf2bub[i][j][1]/88.1))#, cBubFinalAll[i, j], lambdaPure[i, j])
print('\t Mole ratio reactants, HBU/BuOH: %2f' % ((cf2bub[i][j][1]/88.1)/(cf2bub[i][j][0]/74.122),))
print('\t Mixing ratio = (V_HBu/V_tot): %.2f ' % lambdaPure[i, j])#, cBubFinalAll[i, j], lambdaPure[i, j])
print('\t Outlet concentration of BuOH from fermenter: %.2f [g/L]' %buohBlockPureMesh[i][j][0])
print('\t Outlet concentration of HBu from fermenter: %.2f [g/L]' %hbuBlockPureMesh[i][j][1])
print('\t Outlet concentration of S from fermenter: %.2f [g/L], %.2f [g/L]' %((buohBlockPureMesh[i][j][2], hbuBlockPureMesh[i][j][2])))

print('Outlet of reactor (t = %.2f h):' %cPFRall[i][j][-1, -1])
print('\t BuB: %.2f [g/L] <=> %.2f [mmol/L]' % (cPFRall[i][j][-1, 2], 1e3*cPFRall[i][j][-1, 2]/144.21))
print('\t BuOH: %.2f [g/L] <=> %.2f [mmol/L]' % (cPFRall[i][j][-1, 0], 1e3*cPFRall[i][j][-1, 0]/74.122))
print('\t HBu: %.2f [g/L] <=> %.2f [mmol/L]' % (cPFRall[i][j][-1, 1], 1e3*cPFRall[i][j][-1, 1]/88.1))

ax['AR'].plot(cPFRall[i][j][:, 0], cPFRall[i][j][:, 3], cPFRall[i][j][:, 2],
	color = 'r', linewidth = .5)
ax['AR'].scatter(cf2bub[i][j][0], #cfBuOH
	cf2bub[i][j][3], #cfS
	cf2bub[i][j][2], #cfBuB
	color= 'r', marker = '^', s = 30)



#Make a contour plot of the results
fig['AR2'] = plt.figure('AR2')
ax['AR2'] = fig['AR2'].add_subplot(111)
ax['AR2'].set_xlabel(r'$C_{f, BuOH}$ [g/L]')
ax['AR2'].set_ylabel(r'$C_{f, S}$ [g/L]')

myCsMat = np.nan_to_num(cs)
myBuohMat = np.nan_to_num(XmSel)
myContour = ax['AR2'].contourf(myBuohMat, myCsMat, cBubFinalAll, levels = np.arange(0,20,2.5))
ax['AR2'].scatter(cf2bub[i][j][0], cf2bub[i][j][3], s=30, color = 'r', 
	marker = 'x', label = 'Optimal feed')
#print(XmSel)
# Make a colorbar for the ContourSet returned by the contourf call.
cbar = plt.colorbar(myContour)
cbar.ax.set_ylabel(r'$C_{BuB}$ [g/L]')
ax['AR2'].legend()





#produce an AR plot for the optimal case
fig['optAR'] = plt.figure('optAR')	
ax['optAR'] = fig['optAR'].add_subplot(111)
ax['optAR'].set_xlabel(r'$C_{BuB}$ [g/L]')
ax['optAR'].set_ylabel(r'$\tau$ [h]')
ax['optAR'].plot(cPFRall[i][j][:, 2], cPFRall[i][j][:, -1], color = 'r', label = 'PFR')
optCSTR = np.genfromtxt(dirCSV['bub'] + '/cCSTRall-' + str(i) + '-' + str(j) + '.csv', 
	delimiter = ',')
ax['optAR'].scatter(optCSTR[:, 2], optCSTR[:, -1], color = 'b', label = 'CSTR', s = 10)
#Include the convex hull. data gathering
cDataAllBubOpt = np.vstack((cf2bub[i][j][[2, -1]], cPFRall[i][j][:, [2, -1]]))
# Add maximum points (actual max x-value and the y-limit of the current axis, as well as the line 
#from the feed point to max y-lim)
maxXdata = np.amax(cDataAllBubOpt[:, 0])
maxYlim = ax['optAR'].get_ylim()[1]
maximumPoints = np.array([[maxXdata, maxYlim],
	[cf2bub[i][j][2], maxYlim]])
cDataAllBubOpt = np.vstack((cDataAllBubOpt, maximumPoints))#add the new points

hull['optAR'] = artools.convhull(cDataAllBubOpt, incrementalBool = True)
h = ax['optAR'].fill_between(cDataAllBubOpt[hull['optAR'].vertices, 0], 
	cDataAllBubOpt[hull['optAR'].vertices, 1], alpha = 0.2)
ax['optAR'].set_title(r'$\lambda = Q_{HBu}/(Q{HBu} + Q_{BuOH})$ = ' + str('%.2f' %lambdaPure[i][j]))



#save the data to csv files so that it may be used in different plotting software

dirCSV['optARdata'] = os.getcwd() + '/optARdata'
if not os.path.exists(dirCSV['optARdata']):
	os.makedirs(dirCSV['optARdata'])


np.savetxt(dirCSV['optARdata'] + '/dataPFRallf.csv', cPFRall[i][j], delimiter = ',')
np.savetxt(dirCSV['optARdata'] + '/dataCSTRall.csv', optCSTR, delimiter = ',')
np.savetxt(dirCSV['optARdata'] + '/dataCall.csv', cDataAllBubOpt, delimiter = ',')
np.savetxt(dirCSV['optARdata'] + '/hullVertices.csv', hull['optAR'].vertices, delimiter = ',')

"""
Make a quiver plot of the rate vectors on the plot. In this section, we'll work in a X-Y space since
it's two dimensions.

Here: x-dimension is cp, y-dimension is tau
"""
xlim = ax['optAR'].get_xlim()
ylim = ax['optAR'].get_ylim()
npPlt = 15 #number of points in each direction the rate vector will be evaluated in each direction.
#Thus, we'll make a 10x10 plot here
Xp = np.linspace(*xlim, num = npPlt)# *xlim unpacks the data (tuple) in xlim. p stands here for "points"
Yp = np.linspace(*ylim, num = npPlt)

Xm, Ym = np.meshgrid(Xp, Yp)#Make a meshgrid. Don't include the points on the limit of the
#axes on the figure

#compute the rate vector in each point in the meshgrid
VxMesh = np.zeros(Xm.shape) #V stands for velocity in a quiver plot
VyMesh = np.zeros(Ym.shape) 

#indices in arrays
ibuoh = 0
ihbu = 1
ibub = 2
isub = 3
#ihac
#ietoh
itau = 4
#feed concentration, according to indices declared above
cfAll = cf2bub[i][j]#g/L. Feed concentrations (last is tau [h])
cf = np.array([cfAll[ibub], cfAll[itau]])# [g/l, h]. State space vector
MW = np.array([74.122, 88.1, 144.21, 180.1559]) #[g/mol] [buoh, hbu, bub, s]
rc = {'rmax': 0.02*60*160e-3, #mol/(l min g_enzyme) * min/h * g_enz (used 160g enzyeme fr 0,01mol buoh/hbu in 15ml sol)
	'Kma': 0.051, #mol/L
	'Kmb': .228, #mol/L
	'Kia': .033, #mol/L
	'cf': cf, 'cfAll': cfAll, # g/L
	'MW': MW, # g/mol
	'isub': isub, 'ihbu': ihbu,'ibuoh': ibuoh, 'ibub': ibub, 'itau': itau} #indices

for i in range(len(Xm)):
	for j in range(len(Xm[0,:])):
		cxi = Xm[i, j]#Obtain the current concentrations of X and p
		cyi = Ym[i, j]
		#Check for non-negativity constraint on all the concentrations
		cAlli = kineticsBub.solveMoleBalance(np.array([[cxi, cyi]]), rc)
		if np.all(cAlli >= -1e-3): #Check non-negativity constraint	
			ri = kineticsBub.rxRateFun(np.array([cxi, cyi]), rc) #compute the rate vector.
			#Units for the rate vector is dcdt. However, it is not required to multiply the rate vector
			# with a timestep, since the vector is going to be normalized soon anyway
			dcx = ri[0] #rate vector for X (decomposed concentration vector)
			dcy = ri[1] # same for p
			dcxn = dcx/np.sqrt(dcx**2 + dcy**2)#normalized vectors
			dcyn = dcy/np.sqrt(dcx**2 + dcy**2)
			VxMesh[i, j] = dcxn
			VyMesh[i, j] = dcyn
		else:
			VxMesh[i, j] = np.nan 
			VyMesh[i, j] = np.nan 

ax['optAR'].quiver(Xm, Ym, VxMesh, VyMesh, color = 'k', alpha = 0.25)



#set the limits to include only the first quadrant
ax['optAR'].set_xlim([0, ax['optAR'].get_xlim()[1]])
ax['optAR'].set_ylim([0, maxYlim])#ax['optAR'].get_ylim()[1]])
ax['optAR'].legend()

# # Plot the solution of the linear equations (obtain cbuoh and chbu from cs) to check if the 
# # corresponding solutions are on the triangle
# ax['boundary'].scatter(eq['buohFromS'](buoh['boundary'][:, 2]), np.zeros(buoh['boundary'][:, 2].shape),
# 	buoh['boundary'][:, 2], color = 'k', marker = 'o')
# ax['boundary'].scatter(np.zeros(hbu['boundary'][:, 2].shape), eq['hbuFromS'](hbu['boundary'][:, 2]),
# 	hbu['boundary'][:, 2], color = 'k', marker = 'o')


ax['boundary'].legend()
ax['AR'].legend()


makeVideoShow = False
if makeVideoShow:
	for angle in range(0, 40):
	    ax['boundary'].view_init(30, angle)
	    plt.draw()
	    plt.pause(1e-8)





np.savetxt(dirCSV['optARdata'] + '/dataGridX.csv', Xm, delimiter = ',')
np.savetxt(dirCSV['optARdata'] + '/dataGridY.csv', Ym, delimiter = ',')
np.savetxt(dirCSV['optARdata'] + '/dataGridVx.csv', VxMesh, delimiter = ',')
np.savetxt(dirCSV['optARdata'] + '/dataGridVy.csv', VyMesh, delimiter = ',')


for key in fig.keys(): #This procedure probably don't work if the fig['Cvst']-plots are not uncommented
	fig[key].savefig(dirPlots + '/' + str(fig[key].canvas.get_window_title()), 
		format = 'eps')
	fig[key].savefig(dirPlots + '/' + str(fig[key].canvas.get_window_title()) + '.pdf')

plt.show()

