import numpy as np 
import matplotlib as mpl  
mpl.use('Qt5Agg') #works as an interactive backend. There is currently a bug with the backend macosx
# which makes it not possible to pan 3D plots
import matplotlib.pyplot as plt
# import matplotlib.cm #color map
# import matplotlib.patches
# import matplotlib.lines
from mpl_toolkits.mplot3d import Axes3D
import os
import numpy.matlib
import sys
homeDir = os.path.expanduser('~')
sys.path.insert(0, homeDir + '/Dropbox/Masteroppgave/PythonFiles')
import artools

dirMain = os.getcwd()

#Make plot folder
dirPlots = dirMain + '/plots'
if not os.path.exists(dirPlots):
	os.makedirs(dirPlots)


#define dicts where to store info
buoh = {}
hbu = {}


#direction to csv folders
dirCSV = {}
dirCSV['buoh'] = homeDir + '/Dropbox/Masteroppgave/PythonFiles/bioProcessButylButyrate/scenario2-sep blocks/butanol/computedCSV'
dirCSV['hbu'] = homeDir + '/Dropbox/Masteroppgave/PythonFiles/bioProcessButylButyrate/scenario2-sep blocks/hbuBlock/computedCSV'
dirCSV['resBub'] = os.getcwd() + '/resBub'
if not os.path.exists(dirCSV['resBub']):
	os.makedirs(dirCSV['resBub'])

## Import the data. Only use states which are on the boundary of the AR, that is the CSTR + 2nd PFR
buoh['PFR']= np.genfromtxt(dirCSV['buoh'] + '/dataPFRallf.csv', delimiter = ',') 
buoh['CSTR'] = np.genfromtxt(dirCSV['buoh'] + '/dataCSTRall.csv', delimiter = ',')


hbu['PFR']= np.genfromtxt(dirCSV['hbu'] + '/dataPFRall2.csv', delimiter = ',') 
hbu['CSTR'] = np.genfromtxt(dirCSV['hbu'] + '/dataCSTRall.csv', delimiter = ',')
hbu['optimIndex'] = int(np.genfromtxt(dirCSV['hbu'] + '/optimCSTRindex.txt'))

# Make data points along the mixing lines
buoh['mix'] = np.array([(1-i)*buoh['CSTR'][0, :] + i*buoh['CSTR'][-1, :] 
	for i in np.linspace(0,1)]) #CSTR locus is the optimal here, this is a special case
hbu['mix'] = np.array([(1-i)*hbu['CSTR'][0, :] + i*hbu['CSTR'][hbu['optimIndex'], :] 
	for i in np.linspace(0,1)])

## Obtain data for the boundaries of the feed region to bub block
buoh['boundaryAll'] = np.vstack((buoh['mix'], 
	buoh['PFR']))

hbu['boundaryAll'] = np.vstack((hbu['mix'], 
	hbu['PFR']))


#select the data in the boundaries which are of interest
buoh['boundary'] = np.zeros((buoh['boundaryAll'].shape[0], 3)) #[x,s,hbu,buoh,etoh] indices in buondarAll
buoh['boundary'][:, 0] = buoh['boundaryAll'][:, 3] # cBuOH [g/L]
# buoh['boundary'][:, 1] = buoh['boundaryAll'][:, 2] #cHBu [g/L]. Approximate to 0?
buoh['boundary'][:, 2] = buoh['boundaryAll'][:, 1] # cs [g/L]

hbu['boundary'] = np.zeros((hbu['boundaryAll'].shape[0], 3)) #[s,x,hac,hbu,tau]
# hbu['boundary'][:, 0] equals concentration of BuOH, which is already set to 0
hbu['boundary'][:, 1] = hbu['boundaryAll'][:, 3] # cHBu [g/L]
hbu['boundary'][:, 2] = hbu['boundaryAll'][:, 0] #cS [g/L]


#obtain the convex hull for the region
cDataAllInt = np.vstack((buoh['boundary'], hbu['boundary'])) # store all relevant data in same array
hull = {} # dict if several hull are to be computed
hull['s'] = artools.convhull(cDataAllInt) # key 's' refers to cS on z-axis in plot


## make the 2d projection in (c_buoh - c_hbu)-space
projection = {}
projection['s'] = np.array([buoh['boundary'][0, :], #"feed point" for both buoh and hbu
	buoh['boundary'][-1, :], # end point of buoh
	hbu['boundary'][-1, :]]) # end point for hbu
projection['s'][:, -1] = 0 #set cS = 0 in the projection

#Export the projection in a .csv-file. This file will be read by MATLAB, a grid which is
# inside the projection (by using the MATLAB function inpolygon) will be made and the be exported
# to this file again

numPointsGridTot = 15 #Number of points in each direction in the grid in the projection (15x15 points)
np.savetxt(dirCSV['resBub'] + '/projectionBuohHBu.csv', projection['s'], delimiter = ',')
with open(dirCSV['resBub'] + '/numPointsGrid.txt', 'w') as f:
	f.write(str(numPointsGridTot))
f.close()

#Run MATLAB script now (makeGridInProjection.m, name of the file atm)

#Import the grid (obtained from matlab)
Xm = np.genfromtxt(dirCSV['resBub'] + '/Xm.csv', delimiter = ',')
Ym = np.genfromtxt(dirCSV['resBub'] + '/Ym.csv', delimiter = ',')
Zm = np.genfromtxt(dirCSV['resBub'] + '/Zm.csv', delimiter = ',')
XmSel = Xm[1:, 1:] # Select the part of the mesh which is interesting (i.e not along x and y-axis)
YmSel = Ym[1:, 1:]
numPointsGrid = XmSel.shape[0] #know Xm,Ym,Zm are quadratic and of same shape


"""
In order to relate the grid to the other plots, it is required that the mixing ratio, lambda or 
lambdaPure is obtained. Strategy:
	1) Use the equation for the plane/triangle to obtain cS for every point in the grid
	2) Explot the linear relationship between cs and cbuoh/chbu from the outlet of the previous
		block (can show that R^2 = 1). Make a linear regression such that for every cS, it is 
		possible to obtain cbuoh or chbu from the outlet of the previous block (these points have
		no (direct) correlation to the points in the grid)
	3) Calculate the length from the grid point to each of the "pure" components/outlet of previous 
		block. Pythagoras was used for this (cS for each point is constant and known, operates in 2D)
	4) Calculate mixing ratio from the usual equaiton: lambda = dist1/(dist1 + dist2)

	FLAW: The mixing points do not need to reside in the same 2D plane (don't need to have same cS).
		So this is not exactly true, but the easiest way to map the mesh from one plot to another?


"""
# Relate the grid to the roof of the boundary

#data points for the roof (a triangle, requires only 3 data points)
roofS = np.array([buoh['boundary'][0, :],#the same as hbu['boundary'][0,:], namely the feed point
	buoh['boundary'][-1, :],
	hbu['boundary'][-1, :]])

# Corresponding equation for the surface/triangle
eq = {} 
eq['roofS'] = lambda cbuoh, chbu: buoh['boundary'][0, 2]*(1 - cbuoh/buoh['boundary'][-1, 0] - 
	chbu/hbu['boundary'][-1, 1]) #= cSmax*(1- cbuoh/cbuohMax - chbu/chbuMax)

# #Write equations for cBuoh = a1 + b1*cS and cHBu = a2 + b2*cS (they are linear)

# #obtain the linear equations. First, cBuoh = a + b cS
AbuohEq = np.vstack([buoh['boundary'][:, 2], np.ones(len(buoh['boundary'][:, 2]))]).T#follow the example on doc for np.linalg.lstsq
bBuOH, aBuOH = np.linalg.lstsq(AbuohEq, buoh['boundary'][:, 0])[0]
eq['buohFromS'] = lambda concS: aBuOH + bBuOH*concS
#Then, cHBu = a + b*cS
AhbuEq = np.vstack([hbu['boundary'][:, 2], np.ones(len(hbu['boundary'][:, 2]))]).T#follow the example on doc for np.linalg.lstsq
bHBu, aHBu = np.linalg.lstsq(AhbuEq, hbu['boundary'][:, 1])[0]
eq['hbuFromS'] = lambda concS: aHBu + bHBu * concS



#plot the region
fig = {}
ax = {}

fig['boundary'] = plt.figure('boundary')
ax['boundary'] = fig['boundary'].add_subplot(111, projection = '3d')
ax['boundary'].set_xlabel(r'$C_{BuOH}$ [g/L')
ax['boundary'].set_ylabel(r'$C_{HBu}$ [g/L]')
ax['boundary'].set_zlabel(r'$C_S$ [g/L]')

#plot the oundaries
ax['boundary'].plot(buoh['boundary'][:, 0], buoh['boundary'][:, 1], buoh['boundary'][:, 2], 
	c = 'b', label = 'BuOH') # AR for buoh process
ax['boundary'].plot(hbu['boundary'][:, 0], hbu['boundary'][:, 1], hbu['boundary'][:, 2], 
	c = 'g', label = 'HBu') # AR for hbu process

#Plot the convex hull
ax['boundary'].plot_trisurf(cDataAllInt[:, 0], cDataAllInt[:, 1], cDataAllInt[:, 2], color = 'r', 
	edgecolor = None, alpha = .2, triangles = hull['s'].simplices) #plot the convex hull
# Projection of the AR
ax['boundary'].plot_trisurf(projection['s'][:, 0], projection['s'][:, 1], projection['s'][:, 2], 
	color = 'k', alpha = .2)
# Mesh in the projection
ax['boundary'].scatter(XmSel, YmSel, color = 'k', s = 3, marker = 'x')
# Insert the three points defining the triangle/roofS
ax['boundary'].scatter(roofS[:, 0], roofS[:, 1], roofS[:, 2], c = 'k', marker = '^')


#Make the AR plot. Make it in BuOH-S-BuB space
fig['AR'] = plt.figure('AR')
ax['AR'] = fig['AR'].add_subplot(111, projection = '3d')
ax['AR'].set_xlabel(r'$C_{BuOH}$ [g/L')
ax['AR'].set_ylabel(r'$C_S$ [g/L]')
ax['AR'].set_zlabel(r'$C_{BuB}$ [g/L]')

#the feed region
ax['AR'].plot(buoh['boundary'][:, 0], #cBuOH
	buoh['boundary'][:, 2], #cS
	np.zeros(buoh['boundary'][:,0].shape), #cBuB
	color = 'b', label = 'BuOH')

ax['AR'].plot(hbu['boundary'][:, 0], #cBuOH
	hbu['boundary'][:, 2], #cS
	np.zeros(hbu['boundary'][:,0].shape), #cBuB
	color = 'r', label = 'HBu')


cs = np.zeros(XmSel.shape) #store corresponding cs to each feed point to bub
cs.fill(np.nan) #initialize array to be filled with nan
lambdaPure = np.zeros(XmSel.shape) #same here; store lambda parameter for every point in the mesh
lambdaPure.fill(np.nan)
buohBlockPureMesh = {} # store a np.array([cbuohOut(cs), 0, cs]), where cs is the sugar 
# concentration in the grid and cbuohOut(cs) is the corresponding buoh 
# concentration (concentrations resulting from the outlet of the buoh block)
hbuBlockPureMesh = {} #same here; corresponding concentration from the outlet of the hbu-block
cf2bub = {} #save all the feed points to the bub block
cBubFinalAll = [] #store final (end) concentrations for all cf2bub
for i in range(numPointsGrid):
	buohBlockPureMesh[i] = {}
	hbuBlockPureMesh[i] = {}
	cf2bub[i] = {}
	for j in range(numPointsGrid):
		feedToBub = np.array([XmSel[i, j], YmSel[i, j]]) #np.array([buoh, hbu])
		if not np.isnan(feedToBub).any(): #check if any of the numbers in the feed is nan. If they 
		# are, the feed point is invalid (do nothing)
			cs[i, j] = eq['roofS'](feedToBub[0], feedToBub[1]) #obtain cS for a grid point.
			if cs[i, j] < 0:
				# print('Negative cs in grid from roofEq: ' + str(cs[i,j]) + '. Value changed to 0')
				cs[i, j] = 0
			#obtain the pure concentrations of BuOH and HBu in order to determine the lambda parameter (
			# the relative distance of the grid point to each of the sides)
			cBuOHpure = eq['buohFromS'](cs[i, j])
			cHBuPure = eq['hbuFromS'](cs[i, j])
			dist2BuOH = np.sqrt((feedToBub[0] - cBuOHpure)**2 + (feedToBub[1] - 0)**2)
			dist2HBu= np.sqrt((feedToBub[0] - 0)**2 + (feedToBub[1] - cHBuPure)**2)
			lambdaPure[i, j] = dist2BuOH / (dist2BuOH + dist2HBu) #mixing parameter, lambda
			#Check if the procedure is correct: calculate the mesh based on the lambda parameter
			buohPurePoint = np.array([cBuOHpure, 0, cs[i, j]])  
			hbuPurePoint = np.array([0, cHBuPure, cs[i, j]])
			buohBlockPureMesh[i][j] = np.array([cBuOHpure, 0, cs[i, j]])  
			hbuBlockPureMesh[i][j] = np.array([0, cHBuPure, cs[i, j]])
			cMix = (1 - lambdaPure[i, j])*buohPurePoint + lambdaPure[i, j]*hbuPurePoint
			# cMix = (lambdaPure[i, j])*buohPurePoint + (1-lambdaPure[i, j])*hbuPurePoint
			ax['boundary'].scatter(feedToBub[0], feedToBub[1], cs[i, j], color = 'r', marker = 'o')
			ax['boundary'].scatter(cMix[0], cMix[1], cMix[2], color= 'g', marker = '^', s = 30)
			# plt.draw() #check if the cMix procedure is correct (does red and green markers 
			# # come on top of each other at the same time?)
			# plt.pause(1e-5)

			#Map the grid from the 'boundary'-plot to the 'AR'-plot
			cf2bub[i][j] = np.array([cMix[0], cMix[1], 0, cMix[2]]) #[cBuOH, cHBu, cBuB, cS], 
			# this is the feed 2 bub-block
			cfAr = np.array([cMix[0], cMix[2], 0])
			ax['AR'].scatter(cf2bub[i][j][0], #cBuOH
				cf2bub[i][j][3], #cS
				cf2bub[i][j][2], #cBuB
				color= 'g', marker = '^', s = 30)

# # Plot the solution of the linear equations (obtain cbuoh and chbu from cs) to check if the 
# # corresponding solutions are on the triangle
# ax['boundary'].scatter(eq['buohFromS'](buoh['boundary'][:, 2]), np.zeros(buoh['boundary'][:, 2].shape),
# 	buoh['boundary'][:, 2], color = 'k', marker = 'o')
# ax['boundary'].scatter(np.zeros(hbu['boundary'][:, 2].shape), eq['hbuFromS'](hbu['boundary'][:, 2]),
# 	hbu['boundary'][:, 2], color = 'k', marker = 'o')


ax['boundary'].legend()
ax['AR'].legend()


makeVideoShow = False
if makeVideoShow:
	for angle in range(0, 40):
	    ax['boundary'].view_init(30, angle)
	    plt.draw()
	    plt.pause(1e-8)


for key in fig.keys(): #This procedure probably don't work if the fig['Cvst']-plots are not uncommented
	fig[key].savefig(dirPlots + '/' + str(fig[key].canvas.get_window_title()), 
		format = 'eps')
	fig[key].savefig(dirPlots + '/' + str(fig[key].canvas.get_window_title()) + '.pdf')

plt.show()

