import numpy as np 
import sympy
import matplotlib as mpl  
mpl.use('Qt5Agg')
import matplotlib.lines#for manipulating legend entries
import matplotlib.patches#for manipulating legend entries
# mpl.use('macosx')
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
# from scipy.spatial import ConvexHull
from sympy.interactive.printing import init_printing
init_printing(use_unicode = True)#, wrap_line = False, no_global = True)
from sympy.matrices import Matrix, zeros, ones, diag, GramSchmidt
import os
import numpy.matlib
import time
import sys
homeDir = os.path.expanduser('~')
sys.path.insert(0, homeDir + '/Dropbox/Masteroppgave/PythonFiles')#Need to do this in order
#to import artools, which is a file in another directory
import artools
from kineticsHBu import rxRateFun, solveMoleBalance#, rxRateAllNumpy
import scipy.optimize

font = {'size': 12}
mpl.rc('font', **font)

#Make/check folders for storage of the dictionaries which will be computed
dirmain = os.getcwd()
dirCSV = dirmain + '/computedCSV'
if not os.path.exists(dirCSV):
	os.makedirs(dirCSV)
dirPlots = dirmain + '/plots'
if not os.path.exists(dirPlots):
	os.makedirs(dirPlots)

#indices in cfAll and other matrices
s = 0 # substrate (glucose)
x = 1 # biomass
aa = 2 # acetic acid
ba = 3 # butyric acid
tauInd = 4 # residence time

#stoichiometric matrix
#	s   x aa ba H2 CO2 H2O
N = np.matrix([[
	-1, 0, 0, 1, 2, 2, 0], #rx1
	-1, 0, 1, 0, 4, 2, -2]) #rx2
N = N.T

MW = np.array([12.01*6 + 12*1.008 + 6*15.9994, 12.01 + 1.6*1.008 + 0.43*15.9994 + .25*14.0067,
	60.05, 88.1]) # [g/mol]. Molar weights for MW = [MWS, MWX, MWAa, MWBa]

#Declare initial concentrations
cfAll = np.array([50, .1, 0, 0, 0])#[g/L, g/L, g/L, g/L, h]. Feed concentrations. 
#Contains c0 = [cs0, cx0 caa0, cba0, tau0]
cf = np.array([cfAll[s], cfAll[tauInd]]) # [g/L, h]. State space vector, contains cbb and tau

#Declare parameters in the simulation
rc = {'mu_m': 0.48, 'K_d': 0.0024, 'm_S': 0.002, #[1/h]
	'K_I': 372, 'K_S': 1.62, 'P_d': 48.3, #[g/L]
	'Y_Aa': 0.999, 'alpha_Aa': 0.72, 'alpha_Ba': 2.92, 'Y_X': 0.807, 'Y_Ba': 0.978, #[g/g]
	'beta_Aa': 0, 'beta_Ba': 0.057, # [g/(gh)]
	'exponent': 5.18, # [-]
	's_ind': s, 'x_ind': x, 'aa_ind': aa, 'ba_ind': ba, 'tau_ind': tauInd,#indices in cfAll
	'cfAll': cfAll, # [mol/l]
	'MW': MW, #[g/mol], np.matrix of molecular weights
	'N': N} #[-] stoichiometric matrix

tEnd = {'PFR': 30, 'CSTR': 30} #[h]
epsNonNeg = -1e-3 #threshold value for respecting non-negativity constraint in the reactors



#Make dicts for saving the data from AR calculations
cPFR = {}
tPFR = {}
cCSTR = {}
tCSTR = {}
cPFRall = {} #include for all components
cCSTRall = {}

#The key 'f' refers to 'feed'
rxRate = lambda conc: rxRateFun(conc, rc) #conc refers to state space vector, c = [cp, tau]
cPFR['f'], tPFR['f'] = artools.PFRtrajectory(rxRate, cf, tEnd['PFR'], numPoints = 300)#1e3h integration time was found suitable
#for the biomass to reach equilibrium
cPFRall['f'] = solveMoleBalance(cPFR['f'], rc)

#respect non-negativity constraint
cPFR['f'] = cPFR['f'][cPFRall['f'].min(axis=1) >= epsNonNeg, :]
cPFRall['f'] = cPFRall['f'][cPFRall['f'].min(axis=1) >= epsNonNeg, :]
tPFR['f'] = cPFR['f'][:, -1]

cCSTR['f'], tPFR['f'] = artools.CSTRlocusFast(rxRate, cf, tEnd['CSTR'], numPoints = 300)
cCSTRall['f'] = solveMoleBalance(cCSTR['f'], rc)

#respect the non-negativity constraints
cCSTR['f'] = cCSTR['f'][cCSTRall['f'].min(axis=1) >= epsNonNeg, :]
cCSTRall['f'] = cCSTRall['f'][cCSTRall['f'].min(axis=1) >= epsNonNeg, :]
tCSTR['f'] = cCSTR['f'][:, -1]


# #Obtain the CSTR concentrations. Here, we've multiple solutions. Need to run the procedure several 
# #times with different initial conditions. Need also to check if we've multiple occasions of the same
# #concentrations (can've the same solution for different initial guesses)
# cInitGuessVar = 0.27#np.array([20, 30])
# cInitGuess = np.array([0.01, 0.2, 0.2416])

# cOutCSTR, timeTemp = artools.CSTRlocusSetInitialGuess(rxRate, cf, tEnd['CSTR'], 
# 	np.array([cInitGuess[0], tEnd['CSTR']/2]))#Be sure to include eq (long enough tEnd) 

# for i in range(1, len(cInitGuess)):#obtain all the CSTR solutions
# 	cCSTRtemp, tWaste = artools.CSTRlocusSetInitialGuess(rxRate, cf, tEnd['CSTR'], 
# 		np.array([cInitGuess[i], tEnd['CSTR']/2]))#Be sure to include eq (long enough tEnd)
# 	cOutCSTR = np.vstack((cOutCSTR, cCSTRtemp))

# #select only the unique rows. This part of the code is from stackOverflow
# lenAllCSTRsol = len(cOutCSTR)#see how many solutions which'll be discarded
# sorted_idx = np.lexsort(cOutCSTR.T)
# sorted_cOutCSTR = cOutCSTR[sorted_idx, :]
# row_mask = np.append([True],np.any(np.diff(sorted_cOutCSTR,axis=0),1))
# cCSTR['f'] = sorted_cOutCSTR[row_mask]
# #end stackoverflow

# #remove all the rows which contains 'nan'
# cCSTR['f'] = cCSTR['f'][~np.isnan(cCSTR['f']).any(axis=1)] #removes the rows which contains nan





# cCSTRall['f'] = solveMoleBalance(cCSTR['f'], rc)



#start plotting
fig = {}
ax = {}

if True: #wheter or not to make the concentration vs time plots

	#check for more than one solution of the CSTR
	tv, cv, res = artools.residual1DimCSTReq(rxRate, cf[0], 45, 200, numPoints = 100)
	fig['resCSTR'] = plt.figure('resCSTR')
	ax['resCSTR'] = fig['resCSTR'].add_subplot(111, projection = '3d')
	ax['resCSTR'].set_xlabel('t [h]')
	ax['resCSTR'].set_ylabel('cGuess [g/L]')
	ax['resCSTR'].set_zlabel('Residual CSTR design equation')
	ax['resCSTR'].plot_surface(tv,cv,res)
	xl = ax['resCSTR'].get_xlim()
	yl = ax['resCSTR'].get_ylim()
	xv, yv = np.meshgrid(xl, yl)
	sol = np.zeros(xv.shape)
	ax['resCSTR'].plot_surface(xv,yv,sol, alpha = .2)
	ax['resCSTR'].plot_surface(tv,cv,res)
	np.savetxt(dirCSV + '/solCSTRtv.csv', tv, delimiter = ',')
	np.savetxt(dirCSV + '/solCSTRcv.csv', cv, delimiter = ',')
	np.savetxt(dirCSV + '/solCSTRres.csv', res, delimiter = ',')
	plt.close(fig['resCSTR'])


	#Plot concentration as a function of time
	fig['cVst'] = {}
	ax['cVst'] = {}  
	#Make the biomass and substrate plot
	fig['cVst']['X'] = plt.figure('Xvst')
	ax['cVst']['X'] = fig['cVst']['X'].add_subplot(111)
	#ax['cVst']['X'].set_title('Biomass and glucose')
	ax['cVst']['X'].set_xlabel(r'$\tau$ [h]')
	ax['cVst']['X'].set_ylabel('$C_X$ [g/l]', color = 'C4')
	ax['cVst']['X'].plot(tPFR['f'], cPFRall['f'][:, x], c = 'C4', label = 'PFR')
	ax['cVst']['X'].scatter(tCSTR['f'], cCSTRall['f'][:, x], c = 'C4', 
		marker = 'x', s = 15, label = 'CSTR')
	ax['cVst']['S'] = ax['cVst']['X'].twinx()
	ax['cVst']['S'].set_ylabel('$C_S$ [g/l]', color = 'C1')
	ax['cVst']['S'].plot(tPFR['f'], cPFRall['f'][:, s], c = 'C1')
	ax['cVst']['S'].scatter(tCSTR['f'], cCSTRall['f'][:, s], c = 'C1', 
		marker = 'x', s = 15,)
	handlePFR = mpl.lines.Line2D([], [], c = 'k', label = 'PFR')
	handleCSTR = mpl.lines.Line2D([], [], c = 'k', marker = 'x', linewidth = 0, label = 'CSTR')
	ax['cVst']['S'].spines['left'].set_color('C4')
	ax['cVst']['S'].spines['right'].set_color('C1')
	ax['cVst']['X'].tick_params(axis = 'y', colors = 'C4')
	ax['cVst']['S'].tick_params(axis = 'y', colors = 'C1')
	#ax['cVst']['X'].grid(True)
	ax['cVst']['X'].legend(handles = [handlePFR, handleCSTR], loc = 'right')

	#Butyric acid and acetic acid plot
	fig['cVst']['ba'] = plt.figure('bavst')
	ax['cVst']['ba'] = fig['cVst']['ba'].add_subplot(111)
	#ax['cVst']['ba'].set_title('Butyric and acetic acid')#, cg = %.4f' %cInitGuessVar) for making plots of variations in cInitGuess
	ax['cVst']['ba'].set_xlabel(r'$\tau$ [h]')
	ax['cVst']['ba'].set_ylabel('Concentration [g/L]')
	ax['cVst']['ba'].plot(tPFR['f'], cPFRall['f'][:, ba], c = 'C3')
	ax['cVst']['ba'].scatter(tCSTR['f'], cCSTRall['f'][:, ba], c = 'C3', 
		marker = 'x', s = 15)
	ax['cVst']['ba'].plot(tPFR['f'], cPFRall['f'][:, aa], c = 'C8')
	ax['cVst']['ba'].scatter(tCSTR['f'], cCSTRall['f'][:, aa], c = 'C8', 
		marker = 'x', s = 15)
	handleBa = mpl.patches.Patch(color = 'C3', label = 'HBu')
	handleAa = mpl.patches.Patch(color = 'C8', label = 'HAc')
	handleCSTR = mpl.lines.Line2D([], [], c = 'k', linestyle = '', marker = 'x', markersize = 5, 
		label = 'CSTR')
	handlePFR = mpl.lines.Line2D([], [], c = 'k', label = 'PFR')
	#ax['cVst']['ba'].grid(True)
	ax['cVst']['ba'].legend(handles = [handleBa, handleAa, handlePFR, handleCSTR])

	# fig['sVsHbu'] = plt.figure('sVsHbu')
	# ax['sVsHbu'] = fig['sVsHbu'].add_subplot(111)
	# ax['sVsHbu'].set_xlabel(r'$C_S$ [g/L]')
	# ax['sVsHbu'].set_ylabel(r'$C_{HBu}$ [g/L]')
	# ax['sVsHbu'].plot(cPFRall['f'][:, s], cPFRall['f'][:, ba], c = 'r')
	# ax['sVsHbu'].plot(cCSTRall['f'][:, s], cCSTRall['f'][:, ba], c = 'b')
	# # ax['sVsHbu'].plot(cPFRall['f'][:, s], cPFRall[:, hbu])


	# #Butyric acid/R1 conversion plot
	# fig['cVst']['convba'] = plt.figure('convbavst')
	# ax['cVst']['convba'] = fig['cVst']['convba'].add_subplot(111)
	# ax['cVst']['convba'].set_title('Conversion butyric acid')
	# ax['cVst']['convba'].set_xlabel('t [h]')
	# ax['cVst']['convba'].set_ylabel('Conversion butyric acid [-]')
	# ax['cVst']['convba'].plot(tPFR['f'], PFRconvBA, c = 'r', label = 'PFR')
	# ax['cVst']['convba'].scatter(tCSTR['f'], CSTRconvBA , c = 'b', 
	# 	marker = 'x', label = 'CSTR', s = 5)
	# ax['cVst']['convba'].legend()

	# Substrate plot
	# fig['cVst']['P2'] = plt.figure('P2vst')
	# ax['cVst']['P2'] = fig['cVst']['P2'].add_subplot(111)
	# ax['cVst']['P2'].set_title('Butyl butyrate')
	# ax['cVst']['P2'].set_xlabel('t [h]')
	# ax['cVst']['P2'].set_ylabel('$C_{bb/P2}$ [mol/L]')
	# ax['cVst']['P2'].plot(tPFR['f'], cPFR['f'][:, 0], c = 'r', label = 'PFR')
	# ax['cVst']['P2'].scatter(tCSTR['f'], cCSTR['f'][:, 0], c = 'b', 
	# 	marker = 'x', label = 'CSTR', s = 5)
	# ax['cVst']['P2'].legend()

	# # Save the figures
	# for key in fig['cVst'].keys():
	# 	fig['cVst'][key].savefig(dirPlots + '/' + str(ax['cVst'][key].get_title()), 
	# 		format = 'eps')
	# 	fig['cVst'][key].savefig(dirPlots + '/' + str(ax['cVst'][key].get_title()) + '.pdf')
	# 	# fig['cVst'][key].savefig(dirPlots + '/cInitVar/' + str(ax['cVst'][key].get_title()) + '.pdf')
	# 	# plt.close(fig['cVst'][key])



#Make the AR plot
#Decorate
labels = ['C_S', r'\tau']

fig['AR'] = plt.figure('AR')

ax['AR'] = fig['AR'].add_subplot(111)

ax['AR'].set_xlabel('$' + labels[0] + '$' + ' [g/L]')
ax['AR'].set_ylabel('$' + labels[1] + '$' + ' [h]')


#Plot the (kinetic) AR. Start with PFR from the feed point
lineWidthSecImportance = .1 # for plotting reactors which is not part of the AR boundary
ax['AR'].plot(cPFR['f'][:, 0], tPFR['f'], c = 'r', linewidth = lineWidthSecImportance)
ax['AR'].scatter(cCSTR['f'][:, 0], tCSTR['f'], c = 'b', 
	marker = 'x', label = 'CSTR', s = 5)#CSTR from the feed

"""
Make and draw the convex hull. Make sure that the hull is containing a vertical line 
from (max(X),y_data)-(max(X),max(ylim)) and from (0,0)-(0, max(ylim))
Steps: 
	1) Gather all data
	2) Compute the convex hull
	3) Plot the convex hull
	
"""
#data gathering
cDataAll = cf
for key in cPFR.keys():
	cDataAll = np.vstack((cDataAll, cPFR[key]))
for key in cCSTR.keys():
	cDataAll = np.vstack((cDataAll, cCSTR[key]))

# Add maximum points (actual max x-value and the y-limit of the current axis, as well as the line 
#from the feed point to max y-lim)
maxXdata = np.amax(cDataAll[:, 0])
maxYlim = ax['AR'].get_ylim()[1]
maximumPoints = np.array([[maxXdata, maxYlim],
	[cf[0], maxYlim]])
cDataAll = np.vstack((cDataAll, maximumPoints))#add the new points

#computation
hull = artools.convhull(cDataAll, incrementalBool = True)
# print(cCSTR['f'])
makeOptimCSTRindexPlot = False
if makeOptimCSTRindexPlot:
	optimCSTRIndex, fig['optimCSTRIndex'], ax['optimCSTRIndex'] = artools.optimCSTRforTau2D(hull, 
		cCSTR['f'], rxRate, tEnd['PFR'], makePlot = makeOptimCSTRindexPlot)
else:
	optimCSTRIndex = artools.optimCSTRforTau2D(hull, cCSTR['f'], rxRate, tEnd['PFR'], 
		makePlot = makeOptimCSTRindexPlot)

# Make the new PFR trajectory from the optimal CSTR outlet concentration. Also expand the plot
cPFR['2'], tPFR['2'] = artools.PFRtrajectory(rxRate, cCSTR['f'][optimCSTRIndex, :], 
	tEnd['PFR'] - cCSTR['f'][optimCSTRIndex, 1])

cPFRall['2'] = solveMoleBalance(cPFR['2'], rc)
print(cPFRall['2'][0, :])
print(cPFRall['2'][-1, :])

#respect non-negativity constraint
cPFR['2'] = cPFR['2'][cPFRall['2'].min(axis=1) >= epsNonNeg, :]
cPFRall['2'] = cPFRall['2'][cPFRall['2'].min(axis=1) >= epsNonNeg, :]
tPFR['2'] = cPFR['2'][:, -1]

ax['AR'].plot(cPFR['2'][:, 0], cPFR['2'][:, 1], c = 'r', label = 'PFR')

#Include the mixing line
mixingLine = np.vstack((cf, cCSTR['f'][optimCSTRIndex, :]))
ax['AR'].plot(mixingLine[:, 0], mixingLine[:, 1], c = 'k', label = 'Mixing line', linewidth = 0.5)

cDataAll = np.vstack((cDataAll, cPFR['2'], mixingLine))


# # Make PFR trajectory from each CSTR outlet concetration
# for i in range(len(cCSTR['f'][:, 0])):
# 	cnew, waste = artools.PFRtrajectory(rxRate, cCSTR['f'][i, :], tEnd['PFR'])
# 	# print(cnew[:, 1] > maxYlim)
# 	cnew[cnew[:,1] > tEnd['PFR'], -1] = tEnd['PFR']
# 	# print(cnew)
# 	ax['AR'].plot(cnew[:, 0], cnew[:,1], c = 'r', linewidth = .1)

#Set the limits on the figure to include only the first quadrant (only positive values)
ax['AR'].set_xlim((0, ax['AR'].get_xlim()[1]))#set 0 as the first value, keep automatic value for xmax
ax['AR'].set_ylim((0, ax['AR'].get_ylim()[1])) #same for ymax

"""
Make a quiver plot of the rate vectors on the plot. In this section, we'll work in a X-Y space since
it's two dimensions.

Here: x-dimension is cp, y-dimension is tau
"""
xlim = ax['AR'].get_xlim()
ylim = ax['AR'].get_ylim()
npPlt = 15 #number of points in each direction the rate vector will be evaluated in each direction.
#Thus, we'll make a 10x10 plot here
Xp = np.linspace(*xlim, num = npPlt)# *xlim unpacks the data (tuple) in xlim. p stands here for "points"
Yp = np.linspace(*ylim, num = npPlt)

Xm, Ym = np.meshgrid(Xp, Yp)#Make a meshgrid. Don't include the points on the limit of the
#axes on the figure

#compute the rate vector in each point in the meshgrid
VxMesh = np.zeros(Xm.shape) #V stands for velocity in a quiver plot
VyMesh = np.zeros(Ym.shape) 

for i in range(len(Xm)):
	for j in range(len(Xm[0,:])):
		cxi = Xm[i, j]#Obtain the current concentrations of X and p
		cyi = Ym[i, j]
		#Check for non-negativity constraint on all the concentrations
		cAlli = solveMoleBalance(np.array([[cxi, cyi]]), rc)
		if np.all(cAlli >= epsNonNeg): #Check non-negativity constraint	
			ri = rxRate(np.array([cxi, cyi])) #compute the rate vector.
			#Units for the rate vector is dcdt. However, it is not required to multiply the rate vector
			# with a timestep, since the vector is going to be normalized soon anyway
			dcx = ri[0] #rate vector for X (decomposed concentration vector)
			dcy = ri[1] # same for p
			dcxn = dcx/np.sqrt(dcx**2 + dcy**2)#normalized vectors
			dcyn = dcy/np.sqrt(dcx**2 + dcy**2)
			VxMesh[i, j] = dcxn
			VyMesh[i, j] = dcyn
		else:
			VxMesh[i, j] = np.nan 
			VyMesh[i, j] = np.nan 

ax['AR'].quiver(Xm, Ym, VxMesh, VyMesh, color = 'k', alpha = 0.25)

#Compute the convex hull again so that the plot includes all the extreme points

maxXdata = np.amax(cDataAll[:, 0])
maxYlim = ax['AR'].get_ylim()[1]
maximumPoints = np.array([[maxXdata, maxYlim],
	[0, maxYlim]])
cDataAll = np.vstack((cDataAll, maximumPoints))#add the new points
hull = artools.convhull(cDataAll) #this is the true convex hull
#do the plotting
h = ax['AR'].fill_between(cDataAll[hull.vertices,0], cDataAll[hull.vertices,1], maxYlim, 
	alpha = 0.2)
# h = ax['AR'].fill_between(hull.points[:, 0], hull.points[:, 1], maxYlim, 
# 	alpha = 0.2)
h.set_edgecolor(h.get_facecolor()) #sets the edgecolor to the same as the facecolor

#Check the rate vectors on the PFR boundary
dcxPFRn = np.zeros(len(cPFR['f'][:, 0]))
dcyPFRn = np.zeros(len(cPFR['f'][:, 1]))
for i in range(len(cPFR['f'][:, 0])):
	riPFR = rxRate(cPFR['f'][i, :])
	dcxPFR = riPFR[0]
	dcyPFR = riPFR[1]
	dcxPFRn[i] = dcxPFR/np.sqrt(dcxPFR**2 + dcyPFR**2)#normalized vectors	
	dcyPFRn[i] = dcyPFR/np.sqrt(dcxPFR**2 + dcyPFR**2)

#Check the rate vectors on the CSTR boundary
dcxCSTRn = np.zeros(len(cCSTR['f'][:, 0]))
dcyCSTRn = np.zeros(len(cCSTR['f'][:, 1]))
for i in range(len(cCSTR['f'][:, 0])):
	riCSTR = rxRate(cCSTR['f'][i, :])
	dcxCSTR = riCSTR[0]
	dcyCSTR = riCSTR[1]
	dcxCSTRn[i] = dcxCSTR/np.sqrt(dcxCSTR**2 + dcyCSTR**2)#normalized vectors	
	dcyCSTRn[i] = dcyCSTR/np.sqrt(dcxCSTR**2 + dcyCSTR**2)

# #plot these rate vectors
# ax['AR'].quiver(cPFR['f'][:, 0], cPFR['f'][:, 1], dcxPFRn, dcyPFRn, color = 'k', alpha = 1)
# ax['AR'].quiver(cCSTR['f'][:, 0], cCSTR['f'][:, 1], dcxCSTRn, dcyCSTRn, color = 'k', alpha = 1)
# for i in range(1, len(cCSTR['f'][:, 0])):
# 	ax['AR'].plot([cCSTR['f'][0, 0], cCSTR['f'][i, 0]], [cCSTR['f'][0, 1], cCSTR['f'][i, 1]], 'b--')


#Make a concentration vs time plot for all the components given the optimal residence time
fig['optCvst'] = plt.figure('optCvst')
ax['optCvst'] = fig['optCvst'].add_subplot(111)
ax['optCvst'].set_xlabel(r'$\tau$ [h]')
ax['optCvst'].set_ylabel('Concentration [g/L]')
myColors = ['C1', 'C4', 'C8', 'C3']
myLabels = ['S', 'X', 'HAc', 'HBu']
myLegend = []
for i in range(len(cPFRall['f'][0, :])-1):
	ax['optCvst'].plot(np.array([tCSTR['f'][0], tCSTR['f'][optimCSTRIndex]]), # mixing feed-cstr
		np.array([cCSTRall['f'][0, i], cCSTRall['f'][optimCSTRIndex, i]]), 
		color = myColors[i], linestyle = '--')
	ax['optCvst'].scatter(tCSTR['f'][optimCSTRIndex], #outlet CSTR
		cCSTRall['f'][optimCSTRIndex, i], 
		color = myColors[i], marker = 'x', s = 30)

	ax['optCvst'].plot(tPFR['2'], cPFRall['2'][:, i], color = myColors[i]) #PFR from cstr out

	myLegend.append(mpl.patches.Patch(myColors[i], label = str(myLabels[i]), #colors for legend
			FaceColor = myColors[i]))
#attach legends to the different linestyles
myLegend.append(mpl.lines.Line2D([], [], color = 'k', linestyle = '--', label = 'Mixing line'))
myLegend.append(mpl.lines.Line2D([], [], color = 'k', label = 'PFR'))
myLegend.append(mpl.lines.Line2D([], [], color = 'k', marker = 'x', markersize = 5, LineWidth = 0,
	label = 'CSTR'))
ax['optCvst'].legend(handles=myLegend)





#save the data to csv files so that it may be used in different plotting software
np.savetxt(dirCSV + '/dataPFRf.csv', cPFR['f'], delimiter = ',')
np.savetxt(dirCSV + '/dataPFRallf.csv', cPFRall['f'], delimiter = ',')
np.savetxt(dirCSV + '/dataCSTR.csv', cCSTR['f'], delimiter = ',')
np.savetxt(dirCSV + '/dataCSTRall.csv', cCSTRall['f'], delimiter = ',')
with open(dirCSV + '/optimCSTRindex.txt', 'w') as f:
	f.write(str(optimCSTRIndex))
f.close()
np.savetxt(dirCSV + '/dataPFR2.csv', cPFR['2'], delimiter = ',')
np.savetxt(dirCSV + '/dataPFRall2.csv', cPFRall['2'], delimiter = ',')
np.savetxt(dirCSV + '/dataCall.csv', cDataAll, delimiter = ',')
np.savetxt(dirCSV + '/hullVertices.csv', hull.vertices, delimiter = ',')
np.savetxt(dirCSV + '/dataGridX.csv', Xm, delimiter = ',')
np.savetxt(dirCSV + '/dataGridY.csv', Ym, delimiter = ',')
np.savetxt(dirCSV + '/dataGridVx.csv', VxMesh, delimiter = ',')
np.savetxt(dirCSV + '/dataGridVy.csv', VyMesh, delimiter = ',')
# np.savetxt('datarCSTR.csv', rCSTR, delimiter = ',')


ax['AR'].legend(loc = 'upper left').get_frame().set_linewidth(0.5)
# for key in fig.keys(): #This procedure probably don't work if the fig['Cvst']-plots are not uncommented
# 	fig[key].savefig(dirPlots + '/' + str(fig[key].canvas.get_window_title()), 
# 		format = 'eps')
# 	fig[key].savefig(dirPlots + '/' + str(fig[key].canvas.get_window_title()) + '.pdf')
# # # mpldatacursor.datacursor()
plt.show()
plt.close('all')
