clear variables;
B = [0.678 30 0.06 0.75 3.53 0.272 0.3395 2.26 0.793 0.01 0.951 0.0861 0.1 0.01372]; %butanol fermenter coefficient matrix 
C = [0.48 1.62 0.807 0.978 0.999 2.92 0.057 0.72 0]; %butyric acid fermenter and butyl butyrate esterification coefficient matrix 
D = [0.02, 0.051, 0.228, 0.033]; %butyl butyrate esterification coefficient matrix 
D(1) = D(1)*60*160e-3; %[mol/(L min g_enzyme)] -> [mol/(L h)]. They used 160mg of enzyme for 0.01 mol in 15ml solution (xf(5)/((.01/15e-3)*MW.buoh))*160e-3
MW.buoh = 74.122; %[g/mol]
MW.hbu = 88.1;
MW.bub = 144.21;
x0 = [0.1 50 0 0 0 0 0 0.1 50 0 0 0]; %initial values
tspan.buoh = [0, 20];
tspan.combi = [0, 40]; % time span for hbu/bub block

%solve buoh system
[t.buoh,x.buoh] = ode45(@(t,x) kineticSystemBuOH(t,x,B), tspan.buoh, x0);

%update input to bub-block
x0(1:7) = x.buoh(end, 1:7);
%solve bub system
[t.combi,x.combi] = ode45(@(t,x) kineticSystemHBuAndBuB(t,x,C,D, MW), tspan.combi, x0);

[t.combi2,x.combi2] = ode45(@(t,x) kineticSystemHBuAndBuBsimplified(t,x,C,D, MW), tspan.combi, x0);



%% Plot of the products in the separate blocks (HBu, BuOH and BuB)
figure
subplot(2,1,1)
plot(t.buoh,x.buoh(:,5)); %butanol
hold on
grid on
plot(t.buoh,x.buoh(:,10)); %butyric acid
plot(t.buoh,x.buoh(:,12)); %bub
ylabel('Concentration (g/l)')
xlabel('Time  (h)')
legend('BuOH', 'HBu', 'BuB')
title('Butanol block')

subplot(2,1,2)
ax = gca;
plot(t.combi, x.combi(:, 5)) %butanol
hold on
plot(t.combi, x.combi(:, 10)); %butyric acid
plot(t.combi,x.combi(:,12)); %butyl butyrate

ax.ColorOrderIndex = ax.ColorOrderIndex - 3;
plot(t.combi2, x.combi2(:, 5), '--') %butanol
plot(t.combi2, x.combi2(:, 10), '--'); %butyric acid
plot(t.combi2,x.combi2(:,12), '--'); %butyl butyrate


grid on
ylabel('Concentration (g/l)')
xlabel('Time (h)')
legend([{'BuOH'}, {'HBu'}, {'BuB'}], 'box', 'on')
title('HBu and BuB block')
xl = xlim;
xlim([0, xl(end)])

%% Plot of substrate and biomass in the different blocks


figure
subplot(2,1,1)
%BuOH block
l.buoh.bio = plot(t.buoh,x.buoh(:,1)); %biomass 1
grid on
hold on
l.buoh.s = plot(t.buoh,x.buoh(:,2)); %glucose 2
ylabel('Concentration (g/l)')
xlabel('Time (h)')
title('BuOH block')
legend([l.buoh.bio, l.buoh.s], [{'X_1'}, {'S_1'}])

%HBu and BuB block
subplot(2,1,2)
ax = gca;
grid on
hold on
l.c.bio1 = plot(t.combi,x.combi(:,1), '--'); %biomass 1
l.c.s1 = plot(t.combi,x.combi(:,2), '--'); %glucose 1
l.c.bio2 = plot(t.combi, x.combi(:,8)); % biomass 2
l.c.s2 = plot(t.combi, x.combi(:, 9)); % glucose 2

ax.ColorOrderIndex = ax.ColorOrderIndex - 2;
plot(t.combi2, x.combi2(:,8), '-.'); % biomass 2
plot(t.combi2, x.combi2(:, 9), '-.'); % glucose 2ylabel('Concentration (g/l)')
xlabel('Time (h)')
title('HBu and BuB combined')
legend([l.c.bio1, l.c.s1, l.c.bio2, l.c.s2], [{'X_1'}, {'S_1'}, ...
    {'X_2'}, {'S_2'}])


% % to find optimal substrate ratio %I didn't get it working yet
% % s1 + s2 = s --> ratio = s1/s
% ratio=(0.1:0.1:1);
% n=length(ratio);
% x12=cell(1,n); %since ode45 is a variable step solver, you don't know how many t and x values you'll get each time, so the simplest solution would be to save them all in cell arrays 
% allt=cell(1,n);
% BuB=cell(1,n);
% 
% for i=1:n
% [t,x] = ode45(@(t,x) Scenario1(t,x,B,C,D), tspan, x0); % one cell array for each ratio value aka ratio(i)
% allt{i}=t;
% x12{i}=x(:,12); %BuB
% BuB{i}=x12{i}(end); %final BuB conc. (max)
% end
% 
% figure
% plot(ratio,cell2mat(BuB));
% xlabel('ratio')
% ylabel('Maximum BuB concentration')


% figure 
% plot(t,x(:,12)); %butyl butyrate
% grid on
% ylabel('Concentration (g/l)')
% xlabel('Time (h)')
% legend('Butyl butyrate')
