import numpy as np 
import matplotlib as mpl  
mpl.use('Qt5Agg') #works as an interactive backend. There is currently a bug with the backend macosx
# which makes it not possible to pan 3D plots
import matplotlib.pyplot as plt
# import matplotlib.cm #color map
# import matplotlib.patches
# import matplotlib.lines
from mpl_toolkits.mplot3d import Axes3D
import os
import numpy.matlib
import sys
# import os.path
homeDir = os.path.expanduser('~')
sys.path.insert(0, homeDir + '/Dropbox/Masteroppgave/PythonFiles')
import artools
from kineticsBubAndHBu import rxRateFun, solveMoleBalance#, rxRateAllNumpy


# #Make/check folders for storage of the dictionaries which will be computed
# dirmain = os.getcwd()
# dirCSV = dirmain + '/computedCSV'
# if not os.path.exists(dirCSV):
# 	os.makedirs(dirCSV)
# dirPlots = dirmain + '/plots'
# if not os.path.exists(dirPlots):
# 	os.makedirs(dirPlots)


def getARbubAndHBU(feedToReacNetwork, numPointsPFR = 300):
	"""
	Obtain the AR for a given feed.
	INPUT:
	feedToReacNetwork: np.array() containing the feed composition of all components (not just
		indpendent components)
	numPointsPFR: number of points used for calculating the PFR trajectories in the convex
		hull routine (in order to obtain optimal CSTR index)

	OUTPUT:
		cPFRall: dict, ['f'] -> outlet compositions of PFR with feed point given by
			feedToReacNetwork
			['2'] -> outlet composition of PFR with feed from the outlet of the optimal CSTR
		cCSTRall['f'] -> composition locus of outlet CSTR concentrations with 
			feed from feedToReacNetwork
		optimCSTRindex: int. Containing the index in cCSTRall['f'] which makes the AR biggest,
			that is the residence time which makes the reactor critical.
	"""


	#Declare initial data

	#indices in arrays
	ix = 0
	isub = 1
	ihac = 2
	ihbu = 3
	ibuoh = 4
	ibub = 5

	#feed concentration, according to indices declared above
	cfAll = feedToReacNetwork #g/L. Feed concentrations according to indices
	cf = np.array([cfAll[isub], cfAll[ibub]])# [g/L, g/L]. State space vector
	MW = np.array([np.nan, np.nan, np.nan, 88.1, 74.122, 144.21]) #[g/mol] [buoh, hbu, bub], rest is nan

	p = {} #parameters for the two different kinetic models- bub and hbu
	pbub = {'rmax': 0.02*60*160e-3, #mol/(l min g_enzyme) * min/h * g_enz (used 160g enzyeme fr 0,01mol buoh/hbu in 15ml sol)
		'Kma': 0.051, #mol/L
		'Kmb': 0.228, #mol/L
		'Kia': 0.033} #mol/L
	phbu = {'mu_m': 0.48, 'K_d': 0.0024, 'm_S': 0.002, #[1/h]
		'K_I': 372, 'K_S': 1.62, 'P_d': 48.3, #[g/L]
		'Y_Aa': 0.999, 'alpha_Aa': 0.72, 'alpha_Ba': 2.92, 'Y_X': 0.807, 'Y_Ba': 0.978, #[g/gX]
		'beta_Aa': 0, 'beta_Ba': 0.057, # [g/(gX h)]
		'exponent': 5.18} # [-]
	rc = {'phbu': phbu, # dict for parameter for the hbu model.
		'pbub': pbub, # dict for parameter for the bub model.
		'cf': cf, 'cfAll': cfAll, # g/L
		'MW': MW, # g/mol
		'isub': isub, 'ihbu': ihbu,'ibuoh': ibuoh, 'ibub': ibub, 'ix': ix, 'ihac': ihac} #indices
	tEnd = {'PFR': 80, 'CSTR': 1e3}

	#Make dicts for saving the data from AR calculations
	cPFR = {}
	tPFR = {}
	cCSTR = {}
	tCSTR = {}
	cPFRall = {} #include for all components
	cCSTRall = {}

	#Calculated the PFR trajectories and CSTR locus
	rxRate = lambda conc: rxRateFun(conc, rc)
	cPFR['f'], tPFR['f'] = artools.PFRtrajectory(rxRate, cf, tEnd['PFR'], numPoints = 300) 
	cPFRall['f'] = solveMoleBalance(cPFR['f'], rc)

	cCSTR['f'], tCSTR['f'] = artools.CSTRlocusFast(rxRate, cf, tEnd['CSTR'], numPoints = 300)#Be sure to include eq (tend=1e4 is ok
	cCSTRall['f'] = solveMoleBalance(cCSTR['f'], rc)



	"""
	Make and draw the convex hull. Make sure that the hull is containing a vertical line 
	from (max(X),y_data)-(max(X),max(ylim)) and from (0,0)-(0, max(ylim))
	Steps: 
		1) Gather all data
		2) Compute the convex hull
		3) Plot the convex hull
		
	"""
	#data gathering
	cDataAll = cf
	for key in cPFR.keys():
		cDataAll = np.vstack((cDataAll, cPFR[key]))
	for key in cCSTR.keys():
		cDataAll = np.vstack((cDataAll, cCSTR[key]))

	#computation
	hull = artools.convhull(cDataAll, incrementalBool = True)


	makeOptimCSTRindexPlot = False
	numPointsPFRInCSTRoptim = 200
	# if makeOptimCSTRindexPlot:
	# 	optimCSTRIndex, fig['optimCSTRIndex'], ax['optimCSTRIndex'] = artools.optimCSTRfor2DAR(hull, 
	# 		cCSTR['f'], rxRate, tEnd['PFR'], makePlot = makeOptimCSTRindexPlot, 
	# 		numPointsPFR = numPointsPFRInCSTRoptim)
	# else:
	# 	optimCSTRIndex = artools.optimCSTRfor2DAR(hull, cCSTR['f'], rxRate, tEnd['PFR'], 
	# 		makePlot = makeOptimCSTRindexPlot, numPointsPFR = numPointsPFRInCSTRoptim)
	optimCSTRIndex = artools.optimCSTRfor2DAR(hull, cCSTR['f'], rxRate, tEnd['PFR'], 
			makePlot = makeOptimCSTRindexPlot, numPointsPFR = numPointsPFRInCSTRoptim)

	# Calculate the new PFR trajectory from the optimal CSTR outlet concentration
	cPFR['2'], tPFR['2'] = artools.PFRtrajectory(rxRate, cCSTR['f'][optimCSTRIndex, :], 
		tEnd['PFR'] - cCSTR['f'][optimCSTRIndex, 1])
	cPFRall['2'] = solveMoleBalance(cPFR['2'], rc)

	return cPFRall['f'], cCSTRall['f'], optimCSTRIndex, cPFRall['2']


