import numpy as np 
import matplotlib as mpl  
mpl.use('Qt5Agg') #works as an interactive backend. There is currently a bug with the backend macosx
# which makes it not possible to pan 3D plots
import matplotlib.pyplot as plt
# import matplotlib.cm #color map
# import matplotlib.patches
# import matplotlib.lines
from mpl_toolkits.mplot3d import Axes3D
import os, shutil
import numpy.matlib
import sys
homeDir = os.path.expanduser('~')
sys.path.insert(0, homeDir + '/Dropbox/Masteroppgave/PythonFiles')
import artools
import subprocess
import getARfromFeed
dirMain = os.getcwd()
import scipy.interpolate
import scipy.optimize

font = {'size': 12}
mpl.rc('font', **font)

#Make plot folder
dirPlots = dirMain + '/plots'
if not os.path.exists(dirPlots):
	os.makedirs(dirPlots)


#define dicts where to store info
buoh = {}
hbu = {}


#direction to csv folders
dirCSV = {}
dirCSV['buoh'] = '%s%s%s%s' %(homeDir, '/Dropbox/Masteroppgave/PythonFiles/bioProcessButylButyrate',
	'/scenario4-buohSep', 
	'/butanol/computedCSV')

dirCSV['resBub'] = os.getcwd() + '/resBub'
if not os.path.exists(dirCSV['resBub']):
	os.makedirs(dirCSV['resBub'])

dirCSV['bub'] = dirMain + '/cBubForMainARfile'
if not os.path.exists(dirCSV['bub']): # directory for saving the computed bub-concentrations (initiated from the grid points)
	os.makedirs(dirCSV['bub'])

## Import the data. Only use states which are on the boundary of the AR, that is the CSTR + 2nd PFR
buoh['PFR']= np.genfromtxt(dirCSV['buoh'] + '/dataPFRallf.csv', delimiter = ',') 
buoh['CSTR'] = np.genfromtxt(dirCSV['buoh'] + '/dataCSTRall.csv', delimiter = ',')
# data for 2nd (or lower) mixing line 
buoh['mix2Points'] = np.genfromtxt(dirCSV['buoh'] + '/dataMixLine2.csv', delimiter = ',')

# # Make data points along the mixing lines
# buoh['mix'] = np.array([(1-i)*buoh['CSTR'][0, :] + i*buoh['CSTR'][-1, :] 
# 	for i in np.linspace(0,1)]) #CSTR locus is the optimal here, this is a special case

# Make data points along the mixing lines
buoh['mix'] = np.array([(1-i)*buoh['PFR'][0, :] + i*buoh['PFR'][-1, :] 
	for i in np.linspace(0,1)]) #CSTR locus is the optimal here, this is a special case
buoh['mix2All'] = np.array([(1-i)*buoh['mix2Points'][0, :] + i*buoh['mix2Points'][-1, :] 
	for i in np.linspace(0,1)]) # 


## Obtain data for the boundaries of the feed region to bub block
buoh['boundaryAll'] = np.vstack((buoh['mix'], 
	buoh['PFR']))


#select the data in the boundaries which are of interest
buoh['boundary'] = np.zeros((buoh['boundaryAll'].shape[0], 3)) #[x,s,hbu,buoh,etoh] indices in buondarAll
buoh['boundary'][:, 0] = buoh['boundaryAll'][:, 1] # cS [g/L]
buoh['boundary'][:, 1] = buoh['boundaryAll'][:, 3] #cBuOH [g/L]
# buoh['boundary'][:, 2] = buoh['boundaryAll'][:, 1] # cBuB [g/L] (set to 0 by default)
buoh['mix2'] = np.zeros((buoh['mix2All'].shape[0], 3))
buoh['mix2'][:, [0, 1]] = buoh['mix2All'][:, [1, 3]] #set cS and cBuOH, let cBub = 0

freshFeed = np.array([50, # cS [g/L], feed contains only substrate
	0, #cBuOH [g/L]
	0]) # cBuB [g/L]

#obtain the convex hull for the region
cDataAllFeed = np.vstack((freshFeed, buoh['boundary'])) # store all relevant data in same array
hull = {} # dict if several hull are to be computed
hull['feed'] = artools.convhull(cDataAllFeed[:, 0:-1])  


## make the 2d projection in (cS-cBuOH)-space
projection = {}
projection['bub'] = hull['feed'].points


# Export the projection in a .csv-file. This file will be read by MATLAB, a grid which is
# inside the projection (by using the MATLAB function inpolygon) will be made and the be exported
# to this file again

numPointsGridTot = 30 #Number of points in each direction in the grid in the projection (15x15 points)
np.savetxt(dirCSV['resBub'] + '/projectionSBuOH.csv', projection['bub'], delimiter = ',')
with open(dirCSV['resBub'] + '/numPointsGrid.txt', 'w') as f:
	f.write(str(numPointsGridTot))
f.close()

# #Run MATLAB script now (makeGridInProjection.m, name of the file atm)

#Import the grid (obtained from matlab)
Xm = np.genfromtxt(dirCSV['resBub'] + '/Xm.csv', delimiter = ',')
Ym = np.genfromtxt(dirCSV['resBub'] + '/Ym.csv', delimiter = ',')
Zm = np.genfromtxt(dirCSV['resBub'] + '/Zm.csv', delimiter = ',')
XmSel = Xm[1:, 1:] # Select the part of the mesh which is interesting (i.e not along x and y-axis)
YmSel = Ym[1:, 1:]
numPointsGrid = XmSel.shape[0] #know Xm,Ym,Zm are quadratic and of same shape




#plot the region
fig = {}
ax = {}

fig['AR'] = plt.figure('AR')
ax['AR'] = fig['AR'].add_subplot(111, projection = '3d')
ax['AR'].set_xlabel(r'$C_{S}$ [g/L')
ax['AR'].set_ylabel(r'$C_{BuOH}$ [g/L]')
ax['AR'].set_zlabel(r'$C_{BuB}$ [g/L]')
ax['AR'].view_init(azim = 19, elev = 34)

#plot the boundaries

ax['AR'].plot(buoh['boundary'][:, 0], buoh['boundary'][:, 1], buoh['boundary'][:, 2], 
	c = 'b', label = 'BuOH') # AR for buoh process
ax['AR'].scatter(freshFeed[0], freshFeed[1], freshFeed[2], color = 'C1', 
	s = 15, marker = 'x', label = r'$C_{f, S}^{fresh}$')
# Plot the convex hull
ax['AR'].plot_trisurf(cDataAllFeed[:, 0], cDataAllFeed[:, 1], cDataAllFeed[:, 2], color = 'r', 
	edgecolor = None, alpha = .2, triangles = hull['feed'].simplices) #plot the convex hull

# Mesh in the projection
ax['AR'].scatter(XmSel, YmSel, color = 'k', s = 3, marker = 'x')


# #the feed region
# ax['AR'].plot(buoh['boundary'][:, 0], #cS
# 	buoh['boundary'][:, 1], #cBuOH
# 	np.zeros(buoh['boundary'][:,0].shape), #cBuB
# 	color = 'b', label = 'AR BuOH network')

# ax['AR'].scatter(freshFeed[0], freshFeed[1], freshFeed[2], color = 'm', 
# 	s = 15, marker = 'x', label = r'C_{f, S}^{fresh}')

# lambdaPure = np.zeros(XmSel.shape) #same here; store lambda parameter for every point in the mesh
# lambdaPure.fill(np.nan)
cf2bub = {} #save all the feed points to the bub/hbu block
cBubFinalAll = np.zeros((numPointsGrid, numPointsGrid)) #store final (end) concentrations for all cf2bub
cBubFinalAll.fill(np.nan)
cBubBlock = {}
calcBubConc = False #redo calculations for bub-calculations
cPFRall = {}
cPFR2all = {}
cCSTRall = {}
cDataAllBub = np.array([0.1, # cfX [g/L], assume this is always the case
	freshFeed[0], # cfS [g/L]
	0, # cfHAc [g/L]
	0, # cfHBu [g/L], this should be true most of the time (when mixing with fresh feed and mix line, this is true)
	freshFeed[1], # cfBuOH [g/L], this is just a point inside the AR from the butanol reaction network
	freshFeed[2]])
# feedRegion2bub = {}
# feedRegion2bub['buoh'] = np.column_stack((buoh['boundary'][:, 0], buoh['boundary'][:, 1], 
# 	np.zeros(buoh['boundary'][:, 0].shape), buoh['boundary'][:, 2], 
# 	np.zeros(buoh['boundary'][:, 0].shape))) #last is tau, don't have data for this?
# feedRegion2bub['hbu'] = np.column_stack((hbu['boundary'][:, 0], hbu['boundary'][:, 1],
# 	np.zeros(hbu['boundary'][:, 0].shape), hbu['boundary'][:, 2], 
# 	np.zeros(hbu['boundary'][:, 0].shape)))
# cDataAllBub = np.vstack((feedRegion2bub['buoh'], feedRegion2bub['hbu']))

if calcBubConc:
	optimCSTRindex = []
	#delete all the current files in the folder. This is necessary if a different grid size is used
	for the_file in os.listdir(dirCSV['bub']):
		file_path = os.path.join(dirCSV['bub'], the_file)
		try:
			if os.path.isfile(file_path):
				os.unlink(file_path)
			#elif os.path.isdir(file_path): shutil.rmtree(file_path)
		except Exception as e:
			print(e)
else:
	optimCSTRindex = np.genfromtxt(dirCSV['bub'] + '/optimCSTRindex.csv', delimiter = ',')

for i in range(numPointsGrid):
	cf2bub[i] = {}
	cPFRall[i] = {}
	cPFR2all[i] = {}
	cCSTRall[i] = {}

	for j in range(numPointsGrid):
		feedToBub = np.array([XmSel[i, j], YmSel[i, j]]) #np.array([buoh, hbu])
		if not np.isnan(feedToBub).any(): #check if any of the numbers in the feed is nan. If they 
		# are, the feed point is invalid (do nothing)

			# Obtain the lambda parameter- only need to do this is the sugar feed is bigger than 50g/L
			if freshFeed[0] > 50: # The other stream has concentration outside the AR from buoh block
				print('Fresh feed point is outside the AR from buoh block')
	# 			dist2BuOH = np.sqrt((feedToBub[0] - cBuOHpure)**2 + (feedToBub[1] - 0)**2)
	# 			dist2HBu= np.sqrt((feedToBub[0] - 0)**2 + (feedToBub[1] - cHBuPure)**2)
	# 			lambdaPure[i, j] = dist2BuOH / (dist2BuOH + dist2HBu) #mixing parameter, lambda
	# 			#Check if the procedure is correct: calculate the mesh based on the lambda parameter
	# 			buohPurePoint = np.array([cBuOHpure, 0, cs[i, j]])  
	# 			hbuPurePoint = np.array([0, cHBuPure, cs[i, j]])
	# 			buohBlockPureMesh[i][j] = np.array([cBuOHpure, 0, cs[i, j]])  
	# 			hbuBlockPureMesh[i][j] = np.array([0, cHBuPure, cs[i, j]])
	# 			cMix = (1 - lambdaPure[i, j])*buohPurePoint + lambdaPure[i, j]*hbuPurePoint
	# 			# cMix = (lambdaPure[i, j])*buohPurePoint + (1-lambdaPure[i, j])*hbuPurePoint
	# 			ax['feed'].scatter(feedToBub[0], feedToBub[1], cs[i, j], color = 'r', marker = 'o')
	# 			ax['feed'].scatter(cMix[0], cMix[1], cMix[2], color= 'g', marker = '^', s = 30)
	# 			# plt.draw() #check if the cMix procedure is correct (does red and green markers 
	# 			# # come on top of each other at the same time?)
	# 			# plt.pause(1e-5)

	# 			#Map the grid from the 'boundary'-plot to the 'AR'-plot
	# 			cf2bub[i][j] = np.array([cMix[0], cMix[1], 0, cMix[2]]) #[cBuOH, cHBu, cBuB, cS], 
	# 			# this is the feed 2 bub-block
	# 			cfAr = np.array([cMix[0], cMix[2], 0])
	# 			ax['AR'].scatter(cf2bub[i][j][0], #cBuOH
	# 				cf2bub[i][j][3], #cS
	# 				cf2bub[i][j][2], #cBuB
	# 				color= 'g', marker = '^', s = 30)
			else: #the feed point is contained within the AR from buoh block
				cf2bub[i][j] = np.array([0.1, # cfX [g/L], assume this is always the case
					feedToBub[0], # cfS [g/L]
					0, # cfHAc [g/L]
					0, # cfHBu [g/L], this should be true most of the time (when mixing with fresh feed and mix line, this is true)
					feedToBub[1], # cfBuOH [g/L]
					0]) # cfBuB [g/L]
			#Have feed points, do the calculation (or obtain from .csv-files)
			if calcBubConc: # recalcualte and save output to .csv-files. Takes some time
				cPFRall[i][j], cCSTRall[i][j], optimCSTRindexNew, cPFR2all[i][j] = getARfromFeed.getARbubAndHBU(cf2bub[i][j])
				if optimCSTRindexNew == []:
					optimCSTRindexNew = 0
				# print(optimCSTRindexNew)
				optimCSTRindex.append(optimCSTRindexNew)
				# print(optimCSTRindex)
				np.savetxt(dirCSV['bub'] + '/cPFRall-' + str(i) + '-' + str(j) + '.csv', 
					cPFRall[i][j], delimiter = ',')
				np.savetxt(dirCSV['bub'] + '/cCSTRall-' + str(i) + '-' + str(j) + '.csv', 
					cCSTRall[i][j], delimiter = ',')
				np.savetxt(dirCSV['bub'] + '/cPFR2all-' + str(i) + '-' + str(j) + '.csv', 
					cPFR2all[i][j], delimiter = ',')

			else: # Get the data from the CSV files
				cPFRall[i][j] = np.genfromtxt(dirCSV['bub'] + '/cPFRall-' + str(i) + 
					'-' + str(j) + '.csv', delimiter = ',')
				cCSTRall[i][j] = np.genfromtxt(dirCSV['bub'] + '/cCSTRall-' + str(i) + 
					'-' + str(j) + '.csv', delimiter = ',')
				cPFR2all[i][j] = np.genfromtxt(dirCSV['bub'] + '/cPFR2all-' + str(i) + 
					'-' + str(j) + '.csv', delimiter = ',')


			# #Incorporate the PFR data into the cDataAllBub
			cDataAllBub = np.vstack((cDataAllBub, cPFRall[i][j], cCSTRall[i][j], cPFR2all[i][j]))

			# Obtain max concentration of bub
			allCfromBubReacNetwork = np.concatenate((cPFRall[i][j][:, 5], 
				cCSTRall[i][j][:, 5], cPFR2all[i][j][:, 5])) # 5 is index of bub
			cBubFinalAll[i, j] = np.amax(allCfromBubReacNetwork) 

			# # Plot the bub-data (first, the feed data)
			# ax['AR'].scatter(cf2bub[i][j][1], #cfS
			# 	cf2bub[i][j][4], #cfBuOH
			# 	cf2bub[i][j][5], #cfBuB
			# 	color= 'g', marker = '^', s = 30)
			# Plot the bub solution trajectories
			ax['AR'].plot(cPFRall[i][j][:, 1], cPFRall[i][j][:, 4], cPFRall[i][j][:, 5],
				color = 'C2', linewidth = .5)
			ax['AR'].plot(cCSTRall[i][j][:, 1], cCSTRall[i][j][:, 4], cCSTRall[i][j][:, 5],
				color = 'C2', linewidth = .5)
			ax['AR'].plot(cPFR2all[i][j][:, 1], cPFR2all[i][j][:, 4], cPFR2all[i][j][:, 5],
				color = 'C2', linewidth = .5)
			if calcBubConc: # procedure takes time, nice to visualize how long it is left
				plt.draw()
				plt.pause(1e-8)
if calcBubConc: #need to save the optimCSTRindex for the newly calculated data
	np.savetxt(dirCSV['bub'] + '/optimCSTRindex.csv', optimCSTRindex, delimiter = ',')

		
#Obtain and plot the convex hull
hull['AR'] = artools.convhull(cDataAllBub[:, [1, 4, 5]]) # [1,4,5] correspond to [s,buoh,bub]
# ax['AR'].plot_trisurf(cDataAllBub[:, 1], cDataAllBub[:, 4], cDataAllBub[:, 5], color = 'r',
# 	edgecolor = None, alpha = .2, triangles = hull['AR'].simplices)

#Insert a legend for the Bub-block
ax['AR'].plot([np.nan, np.nan], [np.nan, np.nan], [np.nan, np.nan], color = 'C2', 
	label = 'BuB reactor', linewidth = .5)

ax['AR'].legend()


# Obtain the indices of the maximum concentrations
# i,j = np.unravel_index(cBubFinalAll.argmax(), cBubFinalAll.shape)
i,j = np.unravel_index(np.nanargmax(cBubFinalAll), cBubFinalAll.shape)

print(cf2bub[i][j])
print('Optimal feed to bub-block:')
print('\t S: %.2f [g/L]' % cf2bub[i][j][1])
print('\t BuOH: %.2f [g/L] <=> %.2f [mmol/L]' % (cf2bub[i][j][4], 1e3*cf2bub[i][j][4]/74.122))#, cBubFinalAll[i, j], lambdaPure[i, j])
# print('\t Mole ratio reactants, HBU/BuOH: %2f' % ((cf2bub[i][j][1]/88.1)/(cf2bub[i][j][0]/74.122),))
# print('\t Mixing ratio = (V_HBu/V_tot): %.2f ' % lambdaPure[i, j])#, cBubFinalAll[i, j], lambdaPure[i, j])
print('Outlet of reactor:')#' (t = %.2f h):' %cPFRall[i][j][-1, -1])
print('\t BuB: %.2f [g/L] <=> %.2f [mmol/L]' % (cPFRall[i][j][-1, 5], 1e3*cPFRall[i][j][-1, 5]/144.21))
print('\t BuOH: %.2f [g/L] <=> %.2f [mmol/L]' % (cPFRall[i][j][-1, 4], 1e3*cPFRall[i][j][-1, 4]/74.122))
print('\t HBu: %.2f [g/L] <=> %.2f [mmol/L]' % (cPFRall[i][j][-1, 3], 1e3*cPFRall[i][j][-1, 3]/88.1))
print('\t S: %.2f [g/L] ' % cPFRall[i][j][-1, 1])

ax['AR'].plot(cPFRall[i][j][:, 1], cPFRall[i][j][:, 4], cPFRall[i][j][:, 5],
	color = 'r', linewidth = .5)
ax['AR'].scatter(cf2bub[i][j][1], #cfS
	cf2bub[i][j][4], #cfBuOH
	cf2bub[i][j][5], #cfBuB
	color= 'r', marker = 'x', s = 3)


# makeVideoShow = False
# if makeVideoShow:
# 	for angle in range(0, 40):
# 	    ax['AR'].view_init(30, angle)
# 	    plt.draw()
# 	    plt.pause(1e-8)


#produce an AR plot for the optimal case
fig['optAR'] = plt.figure('optAR')	
ax['optAR'] = fig['optAR'].add_subplot(111)
ax['optAR'].set_xlabel(r'$C_{S}$ [g/L]')
ax['optAR'].set_ylabel(r'$C_{BuB}$ [g/L]')
ax['optAR'].plot(cPFRall[i][j][:, 1], cPFRall[i][j][:, 5], color = 'r', linewidth = .5)
optCSTR = np.genfromtxt(dirCSV['bub'] + '/cCSTRall-' + str(i) + '-' + str(j) + '.csv', 
	delimiter = ',')
ax['optAR'].scatter(cCSTRall[i][j][:, 1], cCSTRall[i][j][:, 5], color = 'b', label = 'CSTR', s = 10)
ax['optAR'].plot(cPFR2all[i][j][:, 1], cPFR2all[i][j][:, 5], color = 'r', label = 'PFR')

#Include the convex hull. data gathering, computation and plot
cDataAllBubOpt = np.vstack((cPFRall[i][j], cCSTRall[i][j], cPFR2all[i][j]))
hull['optAR'] = artools.convhull(cDataAllBubOpt[:, [1, 5]], incrementalBool = True)
ax['optAR'].fill_between(cDataAllBubOpt[hull['optAR'].vertices, 1], 
	cDataAllBubOpt[hull['optAR'].vertices, 5], alpha = 0.2)

ax['optAR'].legend()


#Make a contour plot of the results
fig['AR2'] = plt.figure('AR2')
ax['AR2'] = fig['AR2'].add_subplot(111)
ax['AR2'].set_xlabel(r'$C_{f, S}$ [g/L')
ax['AR2'].set_ylabel(r'$C_{f, BuOH}$ [g/L]')

#Obtain the grid points
myCsVecFull = np.linspace(0, np.amax(projection['bub'][:, 0]), num = numPointsGridTot)
myCsVec = myCsVecFull[1:]
myBuohVecFull = np.linspace(0, np.amax(projection['bub'][:, 1]), num = numPointsGridTot)
myBuohVec = myBuohVecFull[1:]


myContour = ax['AR2'].contourf(myCsVec, myBuohVec, cBubFinalAll, levels = np.arange(0,20,2))
# print(cf2bub[i][j][1], cf2bub[i][j][4])
ax['AR2'].scatter(cf2bub[i][j][1], cf2bub[i][j][4], cBubFinalAll[i][j], 'r', marker = 'x')
ax['AR'].scatter(cf2bub[i][j][1], cf2bub[i][j][4], 0, color = 'r', marker = 'x', s = 30)

#print(XmSel)
# Make a colorbar for the ContourSet returned by the contourf call.
cbar = plt.colorbar(myContour)
# print(cbar.ColorbarBase)
cbar.ax.set_ylabel(r'$C_{BuB}$ [g/L]')



#Obtain the mixing ratio/bypass to optimal feeding point
cfSopt, cfBuohopt = cf2bub[i][j][1], cf2bub[i][j][4]
# weightsSpline = np.zeros(buoh['PFR'][::2, 1].shape)
# weightsSpline = np.array([1 if li > len(buoh['PFR'][::2, 1])/2 else 0.1 for li in range(len(buoh['PFR'][::2, 1]))])
# print(weightsSpline)
# print(buoh['PFR'][::2, 1])
# splEq = scipy.interpolate.interp1d(buoh['PFR'][::2, 1], buoh['PFR'][::2, 3], kind = 'quadratic') #eq for buoh PFR trajectory
numPointsInPFR = len(buoh['PFR'][::2, 1])
# print(buoh['PFR'][::2, 1])
print(int(numPointsInPFR/2))
print(buoh['PFR'][-70::2, 1])
splEq = scipy.interpolate.UnivariateSpline(buoh['PFR'][-70::2, 1], 
	buoh['PFR'][-70::2, 3]) #eq for buoh PFR trajectory


cfS, cfBuoh = 50, 0
aMix = (cfBuoh - cfBuohopt)/(cfS - cfSopt) #describe the mixing line, cBuOh = amix + bmix*cS
bMix = cfBuoh - aMix*cfS
cBuohMixLine = lambda cSInput: aMix*cSInput + bMix

#obtain the point on the PFR trajectory (end point of mixing line)
eqMixPFR = lambda csInput: splEq(csInput) - cBuohMixLine(csInput)
cSPFR = scipy.optimize.fsolve(eqMixPFR, 5.5)
cBuohPFR = cBuohMixLine(cSPFR)

distf2g = np.sqrt((cfS - cfSopt)**2 + (cfBuoh - cfBuohopt)**2) #distance feed (buoh) to grid point
distPFR2g = np.sqrt((cfSopt - cSPFR)**2 + (cfBuohopt - cBuohPFR)**2)
lambdaBypass = distf2g/(distf2g + distPFR2g)
print('Mixing ratio, PFRoutlet/(PFRoutlet + bypass): %.2f' %lambdaBypass)

fig1 = plt.figure(10)
plt.plot(buoh['PFR'][:, 1], buoh['PFR'][:, 3])
plt.plot(buoh['PFR'][::2, 1], splEq(buoh['PFR'][::2, 1]), color = 'C3')
# print(buoh['PFR'][::2, 1])
plt.plot(np.linspace(1,40), splEq(np.linspace(1,40)), marker = 'x', color = 'C3')

plt.scatter(cfSopt, cfBuohopt, color = 'r')
plt.scatter(cfS, cfBuoh, color = 'g')
# print(eqMixPFR(np.array([0,50])))
plt.plot(np.array([0,50]), cBuohMixLine(np.array([0,50])))

for key in fig.keys(): #This procedure probably don't work if the fig['Cvst']-plots are not uncommented
	fig[key].savefig(dirPlots + '/' + str(fig[key].canvas.get_window_title()), 
		format = 'eps')
	fig[key].savefig(dirPlots + '/' + str(fig[key].canvas.get_window_title()) + '.pdf')


# subprocess.Popen('open %s' %'ARMatlab.pdf',shell=True)
plt.show()
plt.close('all')

