import numpy as np 
import matplotlib as mpl  
mpl.use('Qt5Agg') #works as an interactive backend. There is currently a bug with the backend macosx
# which makes it not possible to pan 3D plots
import matplotlib.pyplot as plt
# import matplotlib.cm #color map
# import matplotlib.patches
# import matplotlib.lines
from mpl_toolkits.mplot3d import Axes3D
import os
import numpy.matlib
import sys
# import os.path
homeDir = os.path.expanduser('~')
sys.path.insert(0, homeDir + '/Dropbox/Masteroppgave/PythonFiles')
import artools
from kineticsBubAndHBu import rxRateFun, solveMoleBalance, solveMoleBalancePointStateSpace#, rxRateAllNumpy


# #Make/check folders for storage of the dictionaries which will be computed
# dirmain = os.getcwd()
# dirCSV = dirmain + '/computedCSV'
# if not os.path.exists(dirCSV):
# 	os.makedirs(dirCSV)
# dirPlots = dirmain + '/plots'
# if not os.path.exists(dirPlots):
# 	os.makedirs(dirPlots)

#Declare initial data

#indices in arrays
ix = 0
isub = 1
ihac = 2
ihbu = 3
ibuoh = 4
ibub = 5

#feed concentration, according to indices declared above
cfAll = np.array([.1, 9.322, 0, 0, 8.4462, 0])#g/L. Feed concentrations according to indices
cf = np.array([cfAll[isub], cfAll[ibub]])# [g/L, g/L]. State space vector
MW = np.array([np.nan, np.nan, np.nan, 88.1, 74.122, 144.21]) #[g/mol] [buoh, hbu, bub], rest is nan

p = {} #parameters for the two different kinetic models- bub and hbu
pbub = {'rmax': 0.02*60*160e-3, #mol/(l min g_enzyme) * min/h * g_enz (used 160g enzyeme fr 0,01mol buoh/hbu in 15ml sol)
	'Kma': 0.051, #mol/L
	'Kmb': 0.228, #mol/L
	'Kia': 0.033} #mol/L
phbu = {'mu_m': 0.48, 'K_d': 0.0024, 'm_S': 0.002, #[1/h]
	'K_I': 372, 'K_S': 1.62, 'P_d': 48.3, #[g/L]
	'Y_Aa': 0.999, 'alpha_Aa': 0.72, 'alpha_Ba': 2.92, 'Y_X': 0.807, 'Y_Ba': 0.978, #[g/gX]
	'beta_Aa': 0, 'beta_Ba': 0.057, # [g/(gX h)]
	'exponent': 5.18} # [-]
rc = {'phbu': phbu, # dict for parameter for the hbu model.
	'pbub': pbub, # dict for parameter for the bub model.
	'cf': cf, 'cfAll': cfAll, # g/L
	'MW': MW, # g/mol
	'isub': isub, 'ihbu': ihbu,'ibuoh': ibuoh, 'ibub': ibub, 'ix': ix, 'ihac': ihac} #indices
tEnd = {'PFR': 80, 'CSTR': 80}

#Make dicts for saving the data from AR calculations
cPFR = {}
tPFR = {}
cCSTR = {}
tCSTR = {}
cPFRall = {} #include for all components
cCSTRall = {}

#Calculated the PFR trajectories and CSTR locus
rxRate = lambda conc: rxRateFun(conc, rc)
cPFR['f'], tPFR['f'] = artools.PFRtrajectory(rxRate, cf, tEnd['PFR'], numPoints = 300) 
cPFRall['f'] = solveMoleBalance(cPFR['f'], rc)

cCSTR['f'], tCSTR['f'] = artools.CSTRlocusFast(rxRate, cf, tEnd['CSTR'], numPoints = 300)#Be sure to include eq (tend=1e4 is ok
cCSTRall['f'] = solveMoleBalance(cCSTR['f'], rc)

# # Can search for other solutions of the CSTR equation. There are multiple, but a cVst-plot of
# # all components shows that we end up with a negative concentration of HBu
# cCSTR['f2'], tCSTR['f2'] = artools.CSTRlocusSetInitialGuess(rxRate, cf, tEnd['CSTR'], 
# 	np.array([5,32]))#Be sure to include eq (tend=1e4 is ok
# cCSTRall['f2'] = solveMoleBalance(cCSTR['f2'], rc)

#plot the cVst
fig = {}
ax = {}

# cPFRall['new'] = solveMoleBalance(cPFR['new'], rc)

if True:
	#Make cVst plots
	myLabels = ['X', 'S', 'HAc', 'HBu','BuOH', 'BuB', 'PFR', 'CSTR']
	myColors = ['C4', 'C1', 'C8', 'C3', 'C0', 'C2']

	#Plot cVst for main components
	indMainComponents = [ihbu, ibuoh, ibub] #define here which components to be included
	fig['mcVst'] = plt.figure('cVstMainComp')
	ax['mcVst'] = fig['mcVst'].add_subplot(111)
	ax['mcVst'].set_xlabel(r'$\tau$ [h]')
	ax['mcVst'].set_ylabel('Concentration [g/L]')
	# ax['mcVst'].grid(True)
	# colors = mpl.cm.jet(np.linspace(0,1, len(cfAll)-2)) #other colormaps: jet, rainbow, viridis
	myLegendMain = []

	for i in indMainComponents:#myInd:
		lPFR = ax['mcVst'].plot(tPFR['f'], cPFRall['f'][:, i], color = myColors[i])#, color = colors[i])#, marker = 'o', s = 10)
		ax['mcVst'].scatter(tCSTR['f'], cCSTRall['f'][:, i], color = lPFR[0].get_color(), 
			marker = 'x', s = 15)
		# ax['mcVst'].scatter(tCSTR['f2'], cCSTRall['f2'][:, i], color = lPFR[0].get_color(), marker = 'o', s = 10)
		myLegendMain.append(mpl.patches.Patch(lPFR[0].get_color(), label = str(myLabels[i]), 
			FaceColor = lPFR[0].get_color()))
		# j += 1
	myLegendMain.append(mpl.lines.Line2D([], [], color = 'k', label = 'PFR'))
	myLegendMain.append(mpl.lines.Line2D([], [], color = 'k', marker = 'x', markersize = 5, 
		LineWidth = 0, label = 'CSTR'))
	ax['mcVst'].legend(handles=myLegendMain)


	# Plot cVst for intermediate/secondary components 
	indIntermediates = [isub, ix, ihac]
	fig['icVst'] = plt.figure('cVstSecondaryComp')
	ax['icVst'] = fig['icVst'].add_subplot(111)
	ax['icVst'].set_xlabel(r'$\tau$ [h]')
	ax['icVst'].set_ylabel('Concentration [g/L]')
	# ax['icVst'].grid(True)
	# colors = mpl.cm.jet(np.linspace(0,1, len(cfAll)-2)) #other colormaps: jet, rainbow, viridis
	myLegendInt = []

	for i in indIntermediates:
		lPFR = ax['icVst'].plot(tPFR['f'], cPFRall['f'][:, i], color = myColors[i])#, color = colors[i])#, marker = 'o', s = 10)
		ax['icVst'].scatter(tCSTR['f'], cCSTRall['f'][:, i], color = lPFR[0].get_color(), 
			marker = 'x', s = 15)
		# ax['icVst'].scatter(tCSTR['f2'], cCSTRall['f2'][:, i], color = lPFR[0].get_color(), marker = 'o', s = 10)
		myLegendInt.append(mpl.patches.Patch(lPFR[0].get_color(), label = str(myLabels[i]), 
			FaceColor = lPFR[0].get_color()))
		# j += 1
	myLegendInt.append(mpl.lines.Line2D([], [], color = 'k', label = 'PFR'))
	myLegendInt.append(mpl.lines.Line2D([], [], color = 'k', marker = 'x', markersize = 5, 
		LineWidth = 0, label = 'CSTR'))
	ax['icVst'].legend(handles=myLegendInt)



#Plot the AR
fig['AR'] = plt.figure('AR')
ax['AR'] = fig['AR'].add_subplot(111)
ax['AR'].set_xlabel(r'$c_S$ [g/L]')
ax['AR'].set_ylabel(r'$c_{BuB}$ [g/L]')
ax['AR'].plot(cPFR['f'][:, 0], cPFR['f'][:, 1], color = 'r', label = 'PFR', linewidth = .2)
ax['AR'].scatter(cCSTR['f'][:, 0], cCSTR['f'][:, 1], color = 'b', label = 'CSTR', s = 10)



"""
Make and draw the convex hull. Make sure that the hull is containing a vertical line 
from (max(X),y_data)-(max(X),max(ylim)) and from (0,0)-(0, max(ylim))
Steps: 
	1) Gather all data
	2) Compute the convex hull
	3) Plot the convex hull
	
"""
#data gathering
cDataAll = cf
for key in cPFR.keys():
	cDataAll = np.vstack((cDataAll, cPFR[key]))
for key in cCSTR.keys():
	cDataAll = np.vstack((cDataAll, cCSTR[key]))

# maxXdata = np.amax(cDataAll[:, 0])
# maxYlim = tEnd[max(tEnd, key=tEnd.get)]*1.1 #increase the time with extra 10% #ax['AR'].get_ylim()[1]

# maximumPoints = np.array([[maxXdata, maxYlim],
# 	[cf[0], maxYlim]])
# cDataAll = np.vstack((cDataAll, maximumPoints))#add the new points

#computation
hull = artools.convhull(cDataAll, incrementalBool = True)
h = ax['AR'].fill_between(cDataAll[hull.vertices,0], cDataAll[hull.vertices,1], alpha = 0.2)

makeOptimCSTRindexPlot = True
numPointsPFRInCSTRoptim = 300
if makeOptimCSTRindexPlot:
	optimCSTRIndex, fig['optimCSTRIndex'], ax['optimCSTRIndex'] = artools.optimCSTRfor2DAR(hull, 
		cCSTR['f'], rxRate, tEnd['PFR'], makePlot = makeOptimCSTRindexPlot, 
		numPointsPFR = numPointsPFRInCSTRoptim)
else:
	optimCSTRIndex = artools.optimCSTRfor2DAR(hull, cCSTR['f'], rxRate, tEnd['PFR'], 
		makePlot = makeOptimCSTRindexPlot, numPointsPFR = numPointsPFRInCSTRoptim)


# Make the new PFR trajectory from the optimal CSTR outlet concentration. Also expand the plot
cPFR['2'], tPFR['2'] = artools.PFRtrajectory(rxRate, cCSTR['f'][optimCSTRIndex, :], 
	tEnd['PFR'] - cCSTR['f'][optimCSTRIndex, 1])
ax['AR'].plot(cPFR['2'][:, 0], cPFR['2'][:, 1], c = 'r', label = 'PFR')


#Include the mixing line
mixingLine = np.vstack((cf, cCSTR['f'][optimCSTRIndex, :]))
ax['AR'].plot(mixingLine[:, 0], mixingLine[:, 1], c = 'k', label = 'Mixing line', linewidth = 0.7)

cDataAll = np.vstack((cDataAll, cPFR['2'], mixingLine))

# # # Make PFR trajectory from each CSTR outlet concetration
# # for i in range(len(cCSTR['f'][:, 0])):
# # 	cnew, waste = artools.PFRtrajectory(rxRate, cCSTR['f'][i, :], tEnd['PFR'])
# # 	# print(cnew[:, 1] > maxYlim)
# # 	cnew[cnew[:,1] > tEnd['PFR'], -1] = tEnd['PFR']
# # 	# print(cnew)
# # 	ax['AR'].plot(cnew[:, 0], cnew[:,1], c = 'r', linewidth = .1)

#Set the limits on the figure to include only the first quadrant (only positive values)
# ax['AR'].set_xlim((0, ax['AR'].get_xlim()[1]))#set 0 as the first value, keep automatic value for xmax
# ax['AR'].set_ylim((0, ax['AR'].get_ylim()[1])) #same for ymax

"""
Make a quiver plot of the rate vectors on the plot. In this section, we'll work in a X-Y space since
it's two dimensions.

Here: x-dimension is cp, y-dimension is tau
"""
xlim = ax['AR'].get_xlim()
ylim = ax['AR'].get_ylim()
npPlt = 15 #number of points in each direction the rate vector will be evaluated in each direction.
#Thus, we'll make a 10x10 plot here
Xp = np.linspace(*xlim, num = npPlt)# *xlim unpacks the data (tuple) in xlim. p stands here for "points"
Yp = np.linspace(*ylim, num = npPlt)

Xm, Ym = np.meshgrid(Xp, Yp)#Make a meshgrid. Don't include the points on the limit of the
#axes on the figure

#compute the rate vector in each point in the meshgrid
VxMesh = np.zeros(Xm.shape) #V stands for velocity in a quiver plot
VyMesh = np.zeros(Ym.shape) 

for i in range(len(Xm)):
	for j in range(len(Xm[0,:])):
		cxi = Xm[i, j]#Obtain the current concentrations of X and p
		cyi = Ym[i, j]
		#Check for non-negativity constraint on all the concentrations
		cAlli = solveMoleBalancePointStateSpace(np.array([cxi, cyi]), rc)
		if np.all(cAlli >= 0): #Check non-negativity constraint	
			ri = rxRate(np.array([cxi, cyi])) #compute the rate vector.
			#Units for the rate vector is dcdt. However, it is not required to multiply the rate vector
			# with a timestep, since the vector is going to be normalized soon anyway
			dcx = ri[0] #rate vector for X (decomposed concentration vector)
			dcy = ri[1] # same for p
			dcxn = dcx/np.sqrt(dcx**2 + dcy**2)#normalized vectors
			dcyn = dcy/np.sqrt(dcx**2 + dcy**2)
			VxMesh[i, j] = dcxn
			VyMesh[i, j] = dcyn
		else:
			VxMesh[i, j] = np.nan 
			VyMesh[i, j] = np.nan
# 			ax['AR'].scatter(cxi, cyi, color = 'g', marker = 'o', s = 10) 
# ax['AR'].scatter(np.nan, np.nan, color = 'g', marker = 'o', s = 10, label = 'Neg conc')
ax['AR'].quiver(Xm, Ym, VxMesh, VyMesh, color = 'k', alpha = 0.25)

# # #Check the rate vectors on the PFR boundary
# # dcxPFRn = np.zeros(len(cPFR['f'][:, 0]))
# # dcyPFRn = np.zeros(len(cPFR['f'][:, 1]))
# # for i in range(len(cPFR['f'][:, 0])):
# # 	# csi = cf[1] - (rc['Ypx']/rc['Yxs'])*(cPFR['f'][i, 2] - cf[2]) #Check if the function for cs is correct
# # 	# print(csi - cPFR['f'][i, 1])
# # 	riPFR = rxRate(cPFR['f'][i, :])
# # 	dcxPFR = riPFR[0]
# # 	dcyPFR = riPFR[1]
# # 	dcxPFRn[i] = dcxPFR/np.sqrt(dcxPFR**2 + dcyPFR**2)#normalized vectors	
# # 	dcyPFRn[i] = dcyPFR/np.sqrt(dcxPFR**2 + dcyPFR**2)

# # #Check the rate vectors on the CSTR boundary
# # dcxCSTRn = np.zeros(len(cCSTR['f'][:, 0]))
# # dcyCSTRn = np.zeros(len(cCSTR['f'][:, 1]))
# # for i in range(len(cCSTR['f'][:, 0])):
# # 	# csi = cf[1] - (rc['Ypx']/rc['Yxs'])*(cCSTR['f'][i, 2] - cf[2]) #Check if the function for cs is correct
# # 	# print(csi - cPFR['f'][i, 1])
# # 	riCSTR = rxRate(cCSTR['f'][i, :])
# # 	dcxCSTR = riCSTR[0]
# # 	dcyCSTR = riCSTR[1]
# # 	dcxCSTRn[i] = dcxCSTR/np.sqrt(dcxCSTR**2 + dcyCSTR**2)#normalized vectors	
# # 	dcyCSTRn[i] = dcyCSTR/np.sqrt(dcxCSTR**2 + dcyCSTR**2)

# # #plot these rate vectors
# # ax['AR'].quiver(cPFR['f'][:, 0], cPFR['f'][:, 1], dcxPFRn, dcyPFRn, color = 'k', alpha = 1)
# # ax['AR'].quiver(cCSTR['f'][:, 0], cCSTR['f'][:, 1], dcxCSTRn, dcyCSTRn, color = 'k', alpha = 1)
# # for i in range(1, len(cCSTR['f'][:, 0])):
# # 	ax['AR'].plot([cCSTR['f'][0, 0], cCSTR['f'][i, 0]], [cCSTR['f'][0, 1], cCSTR['f'][i, 1]], 'b--')


dirCSV = os.getcwd() + '/optARdata'
if not os.path.exists(dirCSV):
	os.makedirs(dirCSV)
#save the data to csv files so that it may be used in different plotting software
np.savetxt(dirCSV + '/dataPFRf.csv', cPFR['f'], delimiter = ',')
np.savetxt(dirCSV + '/dataCSTR.csv', cCSTR['f'], delimiter = ',')
# np.savetxt(dirCSV + '/optimCSTRindex.txt', np.asarray(optimCSTRIndex), delimiter = ',')
# with open(dirCSV + '/optimCSTRindex.txt', 'w') as f:
# 	f.write(str(optimCSTRIndex))
# f.close()
# np.savetxt(dirCSV + '/dataPFR2.csv', cPFR['2'], delimiter = ',')
np.savetxt(dirCSV + '/dataCall.csv', cDataAll, delimiter = ',')
np.savetxt(dirCSV + '/hullVertices.csv', hull.vertices, delimiter = ',')
np.savetxt(dirCSV + '/dataGridX.csv', Xm, delimiter = ',')
np.savetxt(dirCSV + '/dataGridY.csv', Ym, delimiter = ',')
np.savetxt(dirCSV + '/dataGridVx.csv', VxMesh, delimiter = ',')
np.savetxt(dirCSV + '/dataGridVy.csv', VyMesh, delimiter = ',')
# # np.savetxt('datarCSTR.csv', rCSTR, delimiter = ',')
# np.savetxt(dirCSV + '/solCSTRtv.csv', tv, delimiter = ',')
# np.savetxt(dirCSV + '/solCSTRcv.csv', cv, delimiter = ',')
# np.savetxt(dirCSV + '/solCSTRres.csv', res, delimiter = ',')



# # # for key in fig.keys(): #This procedure probably don't work if the fig['Cvst']-plots are not uncommented
# # # 	fig[key].savefig(dirPlots + '/' + str(fig[key].canvas.get_window_title()), 
# # # 		format = 'eps')
# # # 	fig[key].savefig(dirPlots + '/' + str(fig[key].canvas.get_window_title()) + '.pdf')
# # # mpldatacursor.datacursor()
print(cPFRall['f'][-1, :])
print(cPFR['2'][-1,:])
ax['AR'].legend().get_frame().set_linewidth(0.5)
plt.show()
