import numpy as np 
import matplotlib as mpl  
mpl.use('Qt5Agg') #works as an interactive backend. There is currently a bug with the backend macosx
# which makes it not possible to pan 3D plots
import matplotlib.pyplot as plt
# import matplotlib.cm #color map
# import matplotlib.patches
# import matplotlib.lines
from mpl_toolkits.mplot3d import Axes3D
import os
import numpy.matlib
import sys
# import os.path
homeDir = os.path.expanduser('~')
sys.path.insert(0, homeDir + '/Dropbox/Masteroppgave/PythonFiles')
import artools
from kineticsButanol import rxRateFun, solveMoleBalance#, rxRateAllNumpy


#Make/check folders for storage of the dictionaries which will be computed
dirmain = os.getcwd()
dirCSV = dirmain + '/computedCSV'
if not os.path.exists(dirCSV):
	os.makedirs(dirCSV)
dirPlots = dirmain + '/plots'
if not os.path.exists(dirPlots):
	os.makedirs(dirPlots)

#Declare initial data

#indices in arrays
ix = 0
isub = 1
ihbu = 2
ibuoh = 3
ietoh = 4
itau = 5
#feed concentration, according to indices declared above
cfAll = np.array([0.1, 50, 0, 0, 0, 0])#g/L. Feed concentrations (last is tau [h])
cf = np.array([cfAll[isub], cfAll[ibuoh]])# [g/l, h]. State space vector
#[0.18; 1.2; 0.01; 0.28; 0.41];%[muMax, Kd, kd?, Yxs, Ypx]
B = np.array([0.678, 30, 0.06, 0.75, 3.53, 0.272, 0.3395, 2.26, 0.793,
	0.01, 0.951, 0.0861, 0.1, 0.01372]) #butanol fermenter coefficient matrix )
rc = {'B': B, 'cf': cf, 'cfAll': cfAll, 'ix': ix, 'isub': isub, 'ihbu': ihbu,
	'ibuoh': ibuoh, 'ietoh': ietoh, 'itau': itau} 
tEnd = {'PFR': 20, 'CSTR': 1e2}

#Make dicts for saving the data from AR calculations
cPFR = {}
tPFR = {}
cCSTR = {}
tCSTR = {}
cPFRall = {} #include for all components
cCSTRall = {}

#Calculated the PFR trajectories and CSTR locus
rxRate = lambda conc: rxRateFun(conc, rc)
cPFR['f'], tPFR['f'] = artools.PFRtrajectory(rxRate, cf, tEnd['CSTR'], numPoints = 300) 
cPFRall['f'] = solveMoleBalance(cPFR['f'], rc)

cCSTR['f'], tCSTR['f'] = artools.CSTRlocusFast(rxRate, cf, tEnd['CSTR'], numPoints = 500)#Be sure to include eq (tend=1e4 is ok
cCSTRall['f'] = solveMoleBalance(cCSTR['f'], rc)

## Can search for other solutions of the CSTR equation. There are multiple, but a cVst-plot of
## all components shows that we end up with a negative concentration of HBu
# cCSTR['f2'], tCSTR['f2'] = artools.CSTRlocusSetInitialGuess(rxRate, cf, tEnd['CSTR'], 
# 	np.array([5,10]))#Be sure to include eq (tend=1e4 is ok
# cCSTRall['f2'] = solveMoleBalance(cCSTR['f2'], rc)

#plot the cVst
fig = {}
ax = {}
fig['cVst'] = plt.figure('cVst')
ax['cVst'] = fig['cVst'].add_subplot(111)
ax['cVst'].set_xlabel('t [h]')
ax['cVst'].set_ylabel('Concentration [g/L]')
ax['cVst'].grid(True)
# colors = mpl.cm.jet(np.linspace(0,1, len(cfAll))) #other colormaps: jet, rainbow, viridis
myLabels = ['X', 'S', 'HBu', 'BuOH', 'EtOH', 'PFR', 'CSTR']
myLegend = []

myInd = [isub, ibuoh] # if only the independent species are to be plotted, set cPFR[: myInd[j]]
j = 0
# for i in range(len(cfAll)-1):#myInd:
# 	ax['cVst'].plot(tPFR['f'], cPFRall['f'][:, i], color = colors[i])#, marker = 'o', s = 10)
# 	ax['cVst'].scatter(tCSTR['f'], cCSTRall['f'][:, i], color = colors[i], marker = 'x', s = 15)
# 	# ax['cVst'].scatter(tCSTR['f2'], cCSTRall['f2'][:, i], color = colors[i], marker = 'o', s = 10)
# 	myLegend.append(mpl.patches.Patch(colors[i], label = str(myLabels[i]), FaceColor = colors[i]))
# 	j += 1
# myLegend.append(mpl.lines.Line2D([], [], color = 'k', label = 'PFR'))
# myLegend.append(mpl.lines.Line2D([], [], color = 'k', marker = 'x', markersize = 5, LineWidth = 0,
# 	label = 'CSTR'))
# ax['cVst'].legend(handles=myLegend)

for i in range(len(cfAll)-2):#myInd:
	lPFR = ax['cVst'].plot(tPFR['f'], cPFRall['f'][:, i])#, color = colors[i])#, marker = 'o', s = 10)
	ax['cVst'].scatter(tCSTR['f'], cCSTRall['f'][:, i], color = lPFR[0].get_color(), 
		marker = 'x', s = 15)
	# ax['cVst'].scatter(tCSTR['f2'], cCSTRall['f2'][:, i], color = colors[i], marker = 'o', s = 10)
	myLegend.append(mpl.patches.Patch(lPFR[0].get_color(), label = str(myLabels[i]), 
		FaceColor = lPFR[0].get_color()))
	j += 1
myLegend.append(mpl.lines.Line2D([], [], color = 'k', label = 'PFR'))
myLegend.append(mpl.lines.Line2D([], [], color = 'k', marker = 'x', markersize = 5, LineWidth = 0,
	label = 'CSTR'))
ax['cVst'].legend(handles=myLegend)


#Plot the AR
fig['AR'] = plt.figure('AR')
ax['AR'] = fig['AR'].add_subplot(111)
ax['AR'].set_xlabel(r'$c_S$ [g/L]')
ax['AR'].set_ylabel(r'$c_{BuOH}$ [g/L]')
ax['AR'].plot(cPFR['f'][:, 0], cPFR['f'][:, 1], color = 'r', label = 'PFR')
ax['AR'].scatter(cCSTR['f'][:, 0], cCSTR['f'][:, 1], color = 'b', label = 'CSTR', s = 10)

"""
Make and draw the convex hull. Make sure that the hull is containing a vertical line 
from (max(X),y_data)-(max(X),max(ylim)) and from (0,0)-(0, max(ylim))
Steps: 
	1) Gather all data
	2) Compute the convex hull
	3) Plot the convex hull
	
"""
#data gathering
cDataAll = cf
for key in cPFR.keys():
	cDataAll = np.vstack((cDataAll, cPFR[key]))
for key in cCSTR.keys():
	cDataAll = np.vstack((cDataAll, cCSTR[key]))

# cDataAll = np.vstack((cDataAll, np.array([0, 0])))#add the new points

#computation
hull = artools.convhull(cDataAll, incrementalBool = True)
h = ax['AR'].fill_between(cDataAll[hull.vertices,0], cDataAll[hull.vertices,1], alpha = 0.2)

## Area can't be expanded (rate vector is tangent to CSTR solution at the point of mixing).
## Increase in area indicated by makeOptimCSTRIndexPlot is >0 simply because a cruder PFR
## is simulated (less datapoints). Hence, no PFR can expand the AR

# makeOptimCSTRindexPlot = True
# if makeOptimCSTRindexPlot:
# 	optimCSTRIndex, fig['optimCSTRIndex'], ax['optimCSTRIndex'] = artools.optimCSTRfor2DAR(hull, 
# 		cCSTR['f'], rxRate, tEnd['PFR'], makePlot = makeOptimCSTRindexPlot)
# else:
# 	optimCSTRIndex = artools.optimCSTRfor2DAR(hull, cCSTR['f'], rxRate, tEnd['PFR'], 
# 		makePlot = makeOptimCSTRindexPlot)

# # Make the new PFR trajectory from the optimal CSTR outlet concentration. Also expand the plot
# cPFR['2'], tPFR['2'] = artools.PFRtrajectory(rxRate, cCSTR['f'][optimCSTRIndex, :], tEnd['PFR'])

# cPFRall['2'] = solveMoleBalance(cPFR['2'], rc)

# # #respect non-negativity constraint
# # cPFR['2'] = cPFR['2'][cPFRall['2'].min(axis=1) >= epsNonNeg, :]
# # cPFRall['2'] = cPFRall['2'][cPFRall['2'].min(axis=1) >= epsNonNeg, :]
# # tPFR['2'] = cPFR['2'][:, -1]

# ax['AR'].plot(cPFR['2'][:, 0], cPFR['2'][:, 1], c = 'g', linestyle = '--', label = 'PFR', linewidth = .5)

# #Include the mixing line
# mixingLine = np.vstack((cf, cCSTR['f'][optimCSTRIndex, :]))
# ax['AR'].plot(mixingLine[:, 0], mixingLine[:, 1], c = 'k', label = 'Mixing line', linewidth = 0.5)
# print(optimCSTRIndex)
# cDataAll = np.vstack((cDataAll, cPFR['2'], mixingLine))

# mixingLineTrue = np.vstack((cf, cCSTR['f'][-1, :]))
# ax['AR'].plot(mixingLineTrue[:, 0], mixingLineTrue[:, 1], c = 'k', 
# 	label = 'Mixing line', linewidth = 0.5)

# #Want to find the second mixing line as well
# currentRegion = np.vstack((cPFR['f'], mixingLineTrue)) #dummy variable, need to have cPFR['f'] first in the stack
# hull2 = artools.convhull(currentRegion)

# srtedVertices = np.sort(hull2.vertices)
# startPointMix2 = np.nonzero(srtedVertices - [i for i in range(len(srtedVertices))])[0][0] -1
# mixLine2 = np.vstack((currentRegion[startPointMix2, :], cPFR['f'][-1, :]))
# ax['AR'].plot(mixLine2[:,0], mixLine2[:,1], c = 'k', linewidth = 0.5)
print(np.array([cfAll[:-1]]).shape, cCSTRall['f'].shape)
mixingLineTrue = np.vstack((np.array([cfAll[:-1]]), cCSTRall['f'][-1, :]))
ax['AR'].plot(mixingLineTrue[:, isub], mixingLineTrue[:, ibuoh], c = 'k', 
	label = 'Mixing line', linewidth = 0.5)

#Want to find the second mixing line as well
currentRegion = np.vstack((cPFRall['f'], mixingLineTrue)) #dummy variable, need to have cPFR['f'] first in the stack
hull2 = artools.convhull(currentRegion[:, [isub, ibuoh]])

srtedVertices = np.sort(hull2.vertices)
startPointMix2 = np.nonzero(srtedVertices - [i for i in range(len(srtedVertices))])[0][0] -1
print(startPointMix2)
mixLine2 = np.vstack((currentRegion[startPointMix2, :], cPFRall['f'][-1, :]))
ax['AR'].plot(mixLine2[:,isub], mixLine2[:,ibuoh], c = 'k', linewidth = 0.5)
"""
Make a quiver plot of the rate vectors on the plot. In this section, we'll work in a X-Y space since
it's two dimensions.

Here: x-dimension is cp, y-dimension is tau
"""
xlim = ax['AR'].get_xlim()
ylim = ax['AR'].get_ylim()
npPlt = 15 #number of points in each direction the rate vector will be evaluated in each direction.
#Thus, we'll make a 10x10 plot here
Xp = np.linspace(*xlim, num = npPlt)# *xlim unpacks the data (tuple) in xlim. p stands here for "points"
Yp = np.linspace(*ylim, num = npPlt)

Xm, Ym = np.meshgrid(Xp, Yp)#Make a meshgrid. Don't include the points on the limit of the
#axes on the figure

#compute the rate vector in each point in the meshgrid
VxMesh = np.zeros(Xm.shape) #V stands for velocity in a quiver plot
VyMesh = np.zeros(Ym.shape) 
epsNonNeg = -1e-2
for i in range(len(Xm)):
	for j in range(len(Xm[0,:])):
		cxi = Xm[i, j]#Obtain the current concentrations of X and p
		cyi = Ym[i, j]
		#Check for non-negativity constraint on all the concentrations
		cAlli = solveMoleBalance(np.array([[cxi, cyi]]), rc)
		if np.all(cAlli >= epsNonNeg): #Check non-negativity constraint	
			ri = rxRate(np.array([cxi, cyi])) #compute the rate vector.
			#Units for the rate vector is dcdt. However, it is not required to multiply the rate vector
			# with a timestep, since the vector is going to be normalized soon anyway
			dcx = ri[0] #rate vector for X (decomposed concentration vector)
			dcy = ri[1] # same for p
			dcxn = dcx/np.sqrt(dcx**2 + dcy**2)#normalized vectors
			dcyn = dcy/np.sqrt(dcx**2 + dcy**2)
			VxMesh[i, j] = dcxn
			VyMesh[i, j] = dcyn
		else:
			VxMesh[i, j] = np.nan 
			VyMesh[i, j] = np.nan 

ax['AR'].quiver(Xm, Ym, VxMesh, VyMesh, color = 'k', alpha = 0.25)



#set the limits to include only the first quadrant
ax['AR'].set_xlim([0, ax['AR'].get_xlim()[1]])
ax['AR'].set_ylim([0, ax['AR'].get_ylim()[1]])


for key in fig.keys(): #This procedure probably don't work if the fig['Cvst']-plots are not uncommented
	fig[key].savefig(dirPlots + '/' + str(fig[key].canvas.get_window_title()), 
		format = 'eps')
	fig[key].savefig(dirPlots + '/' + str(fig[key].canvas.get_window_title()) + '.pdf')


plt.show()


#save the data to csv files so that it may be used in different plotting software
np.savetxt(dirCSV + '/dataPFRf.csv', cPFR['f'], delimiter = ',')
np.savetxt(dirCSV + '/dataPFRallf.csv', cPFRall['f'], delimiter = ',')
np.savetxt(dirCSV + '/dataCSTR.csv', cCSTR['f'], delimiter = ',')
np.savetxt(dirCSV + '/dataCSTRall.csv', cCSTRall['f'], delimiter = ',')
np.savetxt(dirCSV + '/dataCall.csv', cDataAll, delimiter = ',')
np.savetxt(dirCSV + '/dataMixLine2.csv', mixLine2, delimiter = ',')
np.savetxt(dirCSV + '/hullVertices.csv', hull.vertices, delimiter = ',')
np.savetxt(dirCSV + '/dataGridX.csv', Xm, delimiter = ',')
np.savetxt(dirCSV + '/dataGridY.csv', Ym, delimiter = ',')
np.savetxt(dirCSV + '/dataGridVx.csv', VxMesh, delimiter = ',')
np.savetxt(dirCSV + '/dataGridVy.csv', VyMesh, delimiter = ',')

